@@ -19,7 +19,7 @@ var bench = common.createBenchmark(main, {
 function main(conf) {
   var n = +conf.n;
   var p = path.win32;
-  var input = '' + conf.path;
+  var input = String(conf.path);
 
   for (var i = 0; i < n; i++) {
     p.parse(input);
