@@ -56,66 +56,73 @@ var controllerObj = (function () {
     }
 
     filterLog = (function () {
-        var parentFilter = { "this": 1, "locals": 1 };
+        var parentFilter = { "this": 1, "locals": 1, "globals": 1 };
         var filter = {};
 
         // Discard all known globals to reduce baseline noise.
         [
             "#__proto__",
-            "NaN",
-            "Infinity",
-            "undefined",
-            "eval",
-            "parseInt",
-            "parseFloat",
-            "isNaN",
-            "isFinite",
+            "Array",
+            "ArrayBuffer",
+            "Atomics",
+            "Boolean",
+            "chWriteTraceEvent",
+            "CollectGarbage",
+            "console",
+            "DataView",
+            "Date",
             "decodeURI",
             "decodeURIComponent",
             "encodeURI",
             "encodeURIComponent",
+            "Error",
             "escape",
-            "unescape",
-            "CollectGarbage",
-            "Object",
-            "Array",
-            "Boolean",
-            "Symbol",
-            "Proxy",
-            "Reflect",
-            "Promise",
-            "Date",
-            "Function",
-            "Math",
-            "Number",
-            "String",
-            "RegExp",
-            "ArrayBuffer",
-            "DataView",
-            "Int8Array",
-            "Uint8Array",
-            "Uint8ClampedArray",
-            "Int16Array",
-            "Uint16Array",
-            "Int32Array",
-            "Uint32Array",
+            "eval",
+            "EvalError",
             "Float32Array",
             "Float64Array",
-            "JSON",
+            "Function",
+            "Infinity",
+            "Int16Array",
+            "Int32Array",
+            "Int8Array",
             "Intl",
+            "isFinite",
+            "isNaN",
+            "JSON",
             "Map",
-            "Set",
-            "WeakMap",
-            "WeakSet",
-            "Error",
-            "EvalError",
+            "Math",
+            "NaN",
+            "Number",
+            "Object",
+            "parseFloat",
+            "parseInt",
+            "print",
+            "Promise",
+            "Proxy",
             "RangeError",
+            "read",
+            "readbuffer",
             "ReferenceError",
+            "Reflect",
+            "RegExp",
+            "Set",
+            "SharedArrayBuffer",
+            "String",
+            "Symbol",
             "SyntaxError",
             "TypeError",
+            "Uint16Array",
+            "Uint32Array",
+            "Uint8Array",
+            "Uint8ClampedArray",
+            "undefined",
+            "unescape",
             "URIError",
+            "WeakMap",
+            "WeakSet",
+            "WebAssembly",
             "WScript",
-            "print"
         ].forEach(function (name) {
             filter[name] = 1;
         });
