@@ -37,7 +37,7 @@ HRESULT __stdcall OnChakraCoreLoadedEntry(TestHooks& testHooks)
     return ChakraRTInterface::OnChakraCoreLoaded(testHooks);
 }
 
-JsRuntimeAttributes jsrtAttributes = JsRuntimeAttributeAllowScriptInterrupt;
+JsRuntimeAttributes jsrtAttributes = JsRuntimeAttributeNone;
 
 int HostExceptionFilter(int exceptionCode, _EXCEPTION_POINTERS *ep)
 {
@@ -158,13 +158,13 @@ void __stdcall PrintVersion()
 }
 
 // On success the param byteCodeBuffer will be allocated in the function.
-HRESULT GetSerializedBuffer(LPCSTR fileContents, JsValueRef *byteCodeBuffer)
+HRESULT GetSerializedBuffer(LPCSTR fileContents, JsFinalizeCallback fileContentFinalizeCallback, JsValueRef *byteCodeBuffer)
 {
     HRESULT hr = S_OK;
 
     JsValueRef scriptSource;
     IfJsErrorFailLog(ChakraRTInterface::JsCreateExternalArrayBuffer((void*)fileContents,
-        (unsigned int)strlen(fileContents), nullptr, nullptr, &scriptSource));
+        (unsigned int)strlen(fileContents), fileContentFinalizeCallback, (void*)fileContents, &scriptSource));
     IfJsErrorFailLog(ChakraRTInterface::JsSerialize(scriptSource, byteCodeBuffer,
         JsParseScriptAttributeNone));
 
@@ -172,7 +172,7 @@ Error:
     return hr;
 }
 
-HRESULT CreateLibraryByteCodeHeader(LPCSTR contentsRaw, DWORD lengthBytes, LPCWSTR bcFullPath, LPCSTR libraryNameNarrow)
+HRESULT CreateLibraryByteCodeHeader(LPCSTR contentsRaw, JsFinalizeCallback contentsRawFinalizeCallback, DWORD lengthBytes, LPCWSTR bcFullPath, LPCSTR libraryNameNarrow)
 {
     HANDLE bcFileHandle = nullptr;
     JsValueRef bufferVal;
@@ -188,21 +188,22 @@ HRESULT CreateLibraryByteCodeHeader(LPCSTR contentsRaw, DWORD lengthBytes, LPCWS
         "#if 0\r\n";
 
 
-    HRESULT hr = GetSerializedBuffer(contentsRaw, &bufferVal);
+    // We still need contentsRaw after this, so pass a null finalizeCallback into it
+    HRESULT hr = GetSerializedBuffer(contentsRaw, nullptr, &bufferVal);
 
-    if (FAILED(hr)) return hr;
+    IfFailedGoLabel((hr), ErrorRunFinalize);
 
-    IfJsrtErrorHR(ChakraRTInterface::JsGetArrayBufferStorage(bufferVal, &bcBuffer, &bcBufferSize));
+    IfJsrtErrorHRLabel(ChakraRTInterface::JsGetArrayBufferStorage(bufferVal, &bcBuffer, &bcBufferSize), ErrorRunFinalize);
 
     bcFileHandle = CreateFile(bcFullPath, GENERIC_WRITE, FILE_SHARE_DELETE,
         nullptr, CREATE_ALWAYS, FILE_ATTRIBUTE_NORMAL, nullptr);
     if (bcFileHandle == INVALID_HANDLE_VALUE)
     {
-        return E_FAIL;
+        IfFailedGoLabel(E_FAIL, ErrorRunFinalize);
     }
 
-    IfFalseGo(WriteFile(bcFileHandle, outputStr, (DWORD)strlen(outputStr), &written, nullptr));
-    IfFalseGo(WriteFile(bcFileHandle, contentsRaw, lengthBytes, &written, nullptr));
+    IfFalseGoLabel(WriteFile(bcFileHandle, outputStr, (DWORD)strlen(outputStr), &written, nullptr), ErrorRunFinalize);
+    IfFalseGoLabel(WriteFile(bcFileHandle, contentsRaw, lengthBytes, &written, nullptr), ErrorRunFinalize);
     if (lengthBytes < 2 || contentsRaw[lengthBytes - 2] != '\r' || contentsRaw[lengthBytes - 1] != '\n')
     {
         outputStr = "\r\n#endif\r\n";
@@ -211,6 +212,13 @@ HRESULT CreateLibraryByteCodeHeader(LPCSTR contentsRaw, DWORD lengthBytes, LPCWS
     {
         outputStr = "#endif\r\n";
     }
+    
+    // We no longer need contentsRaw, so call the finalizer for it if one was provided
+    if(contentsRawFinalizeCallback != nullptr)
+    {
+        contentsRawFinalizeCallback((void*)contentsRaw);
+    }
+
     IfFalseGo(WriteFile(bcFileHandle, outputStr, (DWORD)strlen(outputStr), &written, nullptr));
 
     // Write out the bytecode
@@ -251,6 +259,14 @@ HRESULT CreateLibraryByteCodeHeader(LPCSTR contentsRaw, DWORD lengthBytes, LPCWS
     outputStr = "}\r\n";
     IfFalseGo(WriteFile(bcFileHandle, outputStr, (DWORD)strlen(outputStr), &written, nullptr));
 
+    if(false)
+    {
+ErrorRunFinalize:
+        if(contentsRawFinalizeCallback != nullptr)
+        {
+            contentsRawFinalizeCallback((void*)contentsRaw);
+        }
+    }
 Error:
     if (bcFileHandle != nullptr)
     {
@@ -260,33 +276,26 @@ Error:
     return hr;
 }
 
-static void CALLBACK PromiseContinuationCallback(JsValueRef task, void *callbackState)
-{
-    Assert(task != JS_INVALID_REFERENCE);
-    Assert(callbackState != JS_INVALID_REFERENCE);
-    MessageQueue * messageQueue = (MessageQueue *)callbackState;
-
-    WScriptJsrt::CallbackMessage *msg = new WScriptJsrt::CallbackMessage(0, task);
-    messageQueue->InsertSorted(msg);
-}
+// Used becuase of what's likely a deficiency in the API
+typedef struct {
+    void* scriptBody;
+    JsFinalizeCallback scriptBodyFinalizeCallback;
+    bool freeingHandled;
+} SerializedCallbackInfo;
 
 static bool CHAKRA_CALLBACK DummyJsSerializedScriptLoadUtf8Source(
     JsSourceContext sourceContext,
     JsValueRef* scriptBuffer,
     JsParseScriptAttributes *parseAttributes)
 {
-    const char* script = reinterpret_cast<const char*>(sourceContext);
-    size_t length = strlen(script);
+    SerializedCallbackInfo* serializedCallbackInfo = reinterpret_cast<SerializedCallbackInfo*>(sourceContext);
+    Assert(!serializedCallbackInfo->freeingHandled);
+    serializedCallbackInfo->freeingHandled = true;
+    size_t length = strlen(reinterpret_cast<const char*>(serializedCallbackInfo->scriptBody));
 
     // sourceContext is source ptr, see RunScript below
-    if (ChakraRTInterface::JsCreateExternalArrayBuffer((void*)script, (unsigned int)length,
-        [](void* data)
-        {
-            // sourceContext is source ptr, see RunScript below
-            // source memory was originally allocated with malloc() in
-            // Helpers::LoadScriptFromFile. No longer needed, free() it.
-            free(data);
-        }, nullptr, scriptBuffer) != JsNoError)
+    if (ChakraRTInterface::JsCreateExternalArrayBuffer(serializedCallbackInfo->scriptBody, (unsigned int)length,
+        serializedCallbackInfo->scriptBodyFinalizeCallback, serializedCallbackInfo->scriptBody, scriptBuffer) != JsNoError)
     {
         return false;
     }
@@ -295,16 +304,20 @@ static bool CHAKRA_CALLBACK DummyJsSerializedScriptLoadUtf8Source(
     return true;
 }
 
-HRESULT RunScript(const char* fileName, LPCSTR fileContents, JsValueRef bufferValue, char *fullPath)
+HRESULT RunScript(const char* fileName, LPCSTR fileContents, JsFinalizeCallback fileContentsFinalizeCallback, JsValueRef bufferValue, char *fullPath)
 {
     HRESULT hr = S_OK;
     MessageQueue * messageQueue = new MessageQueue();
     WScriptJsrt::AddMessageQueue(messageQueue);
 
-    IfJsErrorFailLog(ChakraRTInterface::JsSetPromiseContinuationCallback(PromiseContinuationCallback, (void*)messageQueue));
+    IfJsErrorFailLogLabel(ChakraRTInterface::JsSetPromiseContinuationCallback(WScriptJsrt::PromiseContinuationCallback, (void*)messageQueue), ErrorRunFinalize);
 
     if(strlen(fileName) >= 14 && strcmp(fileName + strlen(fileName) - 14, "ttdSentinal.js") == 0)
     {
+        if(fileContentsFinalizeCallback != nullptr)
+        {
+            fileContentsFinalizeCallback((void*)fileContents);
+        }
 #if !ENABLE_TTD
         wprintf(_u("Sential js file is only ok when in TTDebug mode!!!\n"));
         return E_FAIL;
@@ -344,7 +357,7 @@ HRESULT RunScript(const char* fileName, LPCSTR fileContents, JsValueRef bufferVa
                 //handle any uncaught exception by immediately time-traveling to the throwing line in the debugger -- in replay just report and exit
                 if(res == JsErrorCategoryScript)
                 {
-                    wprintf(_u("An unhandled script exception occoured!!!\n"));
+                    wprintf(_u("An unhandled script exception occurred!!!\n"));
 
                     ExitProcess(0);
                 }
@@ -369,27 +382,55 @@ HRESULT RunScript(const char* fileName, LPCSTR fileContents, JsValueRef bufferVa
 
         JsErrorCode runScript;
         JsValueRef fname;
-        IfJsErrorFailLog(ChakraRTInterface::JsCreateString(fullPath,
-            strlen(fullPath), &fname));
+        IfJsErrorFailLogLabel(ChakraRTInterface::JsCreateString(fullPath,
+            strlen(fullPath), &fname), ErrorRunFinalize);
 
         if(bufferValue != nullptr)
         {
-            runScript = ChakraRTInterface::JsRunSerialized(
-                bufferValue,
-                DummyJsSerializedScriptLoadUtf8Source,
-                reinterpret_cast<JsSourceContext>(fileContents),
-                // Use source ptr as sourceContext
-                fname, nullptr /*result*/);
+            if(fileContents == nullptr)
+            {
+                // if we have no fileContents, no worry about freeing them, and the call is simple.
+                runScript = ChakraRTInterface::JsRunSerialized(
+                        bufferValue,
+                        nullptr /*JsSerializedLoadScriptCallback*/,
+                        0 /*SourceContext*/,
+                        fname,
+                        nullptr /*result*/
+                        );
+            }
+            else // fileContents != nullptr
+            {
+                // Memory management is a little more complex here
+                SerializedCallbackInfo serializedCallbackInfo;
+                serializedCallbackInfo.scriptBody = (void*)fileContents;
+                serializedCallbackInfo.scriptBodyFinalizeCallback = fileContentsFinalizeCallback;
+                serializedCallbackInfo.freeingHandled = false;
+
+                // Now we can run our script, with this serializedCallbackInfo as the sourcecontext
+                runScript = ChakraRTInterface::JsRunSerialized(
+                        bufferValue,
+                        DummyJsSerializedScriptLoadUtf8Source,
+                        reinterpret_cast<JsSourceContext>(&serializedCallbackInfo),
+                        // Use source ptr as sourceContext
+                        fname,
+                        nullptr /*result*/);
+                // Now that we're down here, we can free the fileContents if they weren't sent into
+                // a GC-managed object.
+                if(!serializedCallbackInfo.freeingHandled)
+                {
+                    if(fileContentsFinalizeCallback != nullptr)
+                    {
+                        fileContentsFinalizeCallback((void*)fileContents);
+                    }
+                }
+            }
         }
-        else
+        else // bufferValue == nullptr
         {
             JsValueRef scriptSource;
             IfJsErrorFailLog(ChakraRTInterface::JsCreateExternalArrayBuffer((void*)fileContents,
                 (unsigned int)strlen(fileContents),
-                [](void *data)
-                {
-                    free(data);
-                }, nullptr, &scriptSource));
+                fileContentsFinalizeCallback, (void*)fileContents, &scriptSource));
 #if ENABLE_TTD
             if(doTTRecord)
             {
@@ -406,8 +447,10 @@ HRESULT RunScript(const char* fileName, LPCSTR fileContents, JsValueRef bufferVa
                 IfFailedReturn(ChakraRTInterface::JsTTDStart());
             }
 
+            auto sourceContext = WScriptJsrt::GetNextSourceContext();
+            WScriptJsrt::RegisterScriptDir(sourceContext, fullPath);
             runScript = ChakraRTInterface::JsRun(scriptSource,
-                WScriptJsrt::GetNextSourceContext(), fname,
+                sourceContext, fname,
                 JsParseScriptAttributeNone, nullptr /*result*/);
             if (runScript == JsErrorCategoryUsage)
             {
@@ -440,6 +483,14 @@ HRESULT RunScript(const char* fileName, LPCSTR fileContents, JsValueRef bufferVa
         }
     }
 
+    if(false)
+    {
+ErrorRunFinalize:
+        if(fileContentsFinalizeCallback != nullptr)
+        {
+            fileContentsFinalizeCallback((void*)fileContents);
+        }
+    }
 Error:
 #if ENABLE_TTD
     if(doTTRecord)
@@ -489,32 +540,42 @@ Error:
     return hr;
 }
 
-HRESULT CreateAndRunSerializedScript(const char* fileName, LPCSTR fileContents, char *fullPath)
+HRESULT CreateAndRunSerializedScript(const char* fileName, LPCSTR fileContents, JsFinalizeCallback fileContentsFinalizeCallback, char *fullPath)
 {
     HRESULT hr = S_OK;
     JsRuntimeHandle runtime = JS_INVALID_RUNTIME_HANDLE;
     JsContextRef context = JS_INVALID_REFERENCE, current = JS_INVALID_REFERENCE;
     JsValueRef bufferVal;
 
-    IfFailGo(GetSerializedBuffer(fileContents, &bufferVal));
+    // We don't want this to free fileContents when it completes, so the finalizeCallback is nullptr
+    IfFailedGoLabel(GetSerializedBuffer(fileContents, nullptr, &bufferVal), ErrorRunFinalize);
 
     // Bytecode buffer is created in one runtime and will be executed on different runtime.
 
-    IfFailGo(CreateRuntime(&runtime));
+    IfFailedGoLabel(CreateRuntime(&runtime), ErrorRunFinalize);
     chRuntime = runtime;
 
-    IfJsErrorFailLog(ChakraRTInterface::JsCreateContext(runtime, &context));
-    IfJsErrorFailLog(ChakraRTInterface::JsGetCurrentContext(&current));
-    IfJsErrorFailLog(ChakraRTInterface::JsSetCurrentContext(context));
+    IfJsErrorFailLogLabel(ChakraRTInterface::JsCreateContext(runtime, &context), ErrorRunFinalize);
+    IfJsErrorFailLogLabel(ChakraRTInterface::JsGetCurrentContext(&current), ErrorRunFinalize);
+    IfJsErrorFailLogLabel(ChakraRTInterface::JsSetCurrentContext(context), ErrorRunFinalize);
 
     // Initialized the WScript object on the new context
     if (!WScriptJsrt::Initialize())
     {
-        IfFailGo(E_FAIL);
+        IfFailedGoLabel(E_FAIL, ErrorRunFinalize);
     }
 
-    IfFailGo(RunScript(fileName, fileContents, bufferVal, fullPath));
+    // This is our last call to use fileContents, so pass in the finalizeCallback
+    IfFailGo(RunScript(fileName, fileContents, fileContentsFinalizeCallback, bufferVal, fullPath));
 
+    if(false)
+    {
+ErrorRunFinalize:
+        if(fileContentsFinalizeCallback != nullptr)
+        {
+            fileContentsFinalizeCallback((void*)fileContents);
+        }
+    }
 Error:
     if (current != JS_INVALID_REFERENCE)
     {
@@ -557,7 +618,7 @@ HRESULT ExecuteTest(const char* fileName)
         IfJsErrorFailLog(ChakraRTInterface::JsTTDCreateContext(runtime, true, &context));
         IfJsErrorFailLog(ChakraRTInterface::JsSetCurrentContext(context));
 
-        IfFailGo(RunScript(fileName, fileContents, nullptr, nullptr));
+        IfFailGo(RunScript(fileName, fileContents, WScriptJsrt::FinalizeFree, nullptr, nullptr));
 
         unsigned int rcount = 0;
         IfJsErrorFailLog(ChakraRTInterface::JsSetCurrentContext(nullptr));
@@ -652,14 +713,7 @@ HRESULT ExecuteTest(const char* fileName)
             IfFailGo(E_FAIL);
         }
 
-        // canonicalize that path name to lower case for the profile storage
-        // REVIEW: Assuming no utf8 characters here
         len = strlen(fullPath);
-        for (size_t i = 0; i < len; i++)
-        {
-            fullPath[i] = (char)tolower(fullPath[i]);
-        }
-
         if (HostConfigFlags::flags.GenerateLibraryByteCodeHeaderIsEnabled)
         {
             if (HostConfigFlags::flags.GenerateLibraryByteCodeHeader != nullptr && *HostConfigFlags::flags.GenerateLibraryByteCodeHeader != _u('\0'))
@@ -668,7 +722,7 @@ HRESULT ExecuteTest(const char* fileName)
                 CHAR ext[_MAX_EXT];
                 _splitpath_s(fullPath, NULL, 0, NULL, 0, libraryName, _countof(libraryName), ext, _countof(ext));
 
-                IfFailGo(CreateLibraryByteCodeHeader(fileContents, lengthBytes, HostConfigFlags::flags.GenerateLibraryByteCodeHeader, libraryName));
+                IfFailGo(CreateLibraryByteCodeHeader(fileContents, WScriptJsrt::FinalizeFree, lengthBytes, HostConfigFlags::flags.GenerateLibraryByteCodeHeader, libraryName));
             }
             else
             {
@@ -678,11 +732,11 @@ HRESULT ExecuteTest(const char* fileName)
         }
         else if (HostConfigFlags::flags.SerializedIsEnabled)
         {
-            CreateAndRunSerializedScript(fileName, fileContents, fullPath);
+            CreateAndRunSerializedScript(fileName, fileContents, WScriptJsrt::FinalizeFree, fullPath);
         }
         else
         {
-            IfFailGo(RunScript(fileName, fileContents, nullptr, fullPath));
+            IfFailGo(RunScript(fileName, fileContents, WScriptJsrt::FinalizeFree, nullptr, fullPath));
         }
     }
 Error:
@@ -791,6 +845,8 @@ int _cdecl RunJITServer(int argc, __in_ecount(argc) LPWSTR argv[])
     ChakraRTInterface::ArgInfo argInfo = { argc, argv, PrintUsage, nullptr };
     HINSTANCE chakraLibrary = nullptr;
     bool success = ChakraRTInterface::LoadChakraDll(&argInfo, &chakraLibrary);
+    int status = 0;
+    JsInitializeJITServerPtr initRpcServer = nullptr;
 
     if (!success)
     {
@@ -799,25 +855,27 @@ int _cdecl RunJITServer(int argc, __in_ecount(argc) LPWSTR argv[])
     }
 
     UUID connectionUuid;
-    DWORD status = UuidFromStringW((RPC_WSTR)connectionUuidString, &connectionUuid);
+    status = UuidFromStringW((RPC_WSTR)connectionUuidString, &connectionUuid);
     if (status != RPC_S_OK)
     {
-        return status;
+        goto cleanup;
     }
 
-    JsInitializeJITServerPtr initRpcServer = (JsInitializeJITServerPtr)GetProcAddress(chakraLibrary, "JsInitializeJITServer");
-    HRESULT hr = initRpcServer(&connectionUuid, nullptr, nullptr);
-    if (FAILED(hr))
+    initRpcServer = (JsInitializeJITServerPtr)GetProcAddress(chakraLibrary, "JsInitializeJITServer");
+    status = initRpcServer(&connectionUuid, nullptr, nullptr);
+    if (FAILED(status))
     {
-        wprintf(L"InitializeJITServer failed by 0x%x\n", hr);
-        return hr;
+        wprintf(L"InitializeJITServer failed by 0x%x\n", status);
+        goto cleanup;
     }
+    status = 0;
 
+cleanup:
     if (chakraLibrary)
     {
         ChakraRTInterface::UnloadChakraDll(chakraLibrary);
     }
-    return 0;
+    return status;
 }
 #endif
 
@@ -835,6 +893,7 @@ int main(int argc, char** c_argv)
 // xplat-todo: PAL free CH ?
     PAL_InitializeChakraCore();
 #endif
+    int origargc = argc; // store for clean-up later
     argv = new char16*[argc];
     for (int i = 0; i < argc; i++)
     {
@@ -849,22 +908,32 @@ int _cdecl wmain(int argc, __in_ecount(argc) LPWSTR argv[])
 #ifdef _WIN32
     bool runJITServer = HandleJITServerFlag(argc, argv);
 #endif
+    int retval = -1;
+    HRESULT exitCode = E_FAIL;
+    int cpos = 1;
+    HINSTANCE chakraLibrary = nullptr;
+    bool success = false;
+    ChakraRTInterface::ArgInfo argInfo;
+#ifdef _WIN32
+    ATOM lock;
+#endif
 
     if (argc < 2)
     {
         PrintUsage();
         PAL_Shutdown();
-        return EXIT_FAILURE;
+        retval = EXIT_FAILURE;
+        goto return_cleanup;
     }
 
 #ifdef _WIN32
     if (runJITServer)
     {
-        return RunJITServer(argc, argv);
+        retval = RunJITServer(argc, argv);
+        goto return_cleanup;
     }
 #endif
 
-    int cpos = 1;
     for(int i = 1; i < argc; ++i)
     {
         const wchar *arg = argv[i];
@@ -894,7 +963,8 @@ int _cdecl wmain(int argc, __in_ecount(argc) LPWSTR argv[])
         {
             PrintVersion();
             PAL_Shutdown();
-            return EXIT_SUCCESS;
+            retval = EXIT_SUCCESS;
+            goto return_cleanup;
         }
         else if (
 #if !defined(ENABLE_DEBUG_CONFIG_OPTIONS) // release builds can display some kind of help message
@@ -906,7 +976,8 @@ int _cdecl wmain(int argc, __in_ecount(argc) LPWSTR argv[])
         {
             PrintUsage();
             PAL_Shutdown();
-            return EXIT_SUCCESS;
+            retval = EXIT_SUCCESS;
+            goto return_cleanup;
         }
         else if(wcsstr(argv[i], _u("-TTRecord=")) == argv[i])
         {
@@ -937,7 +1008,9 @@ int _cdecl wmain(int argc, __in_ecount(argc) LPWSTR argv[])
         }
         else
         {
+            wchar *temp = argv[cpos];
             argv[cpos] = argv[i];
+            argv[i] = temp;
             cpos++;
         }
     }
@@ -955,15 +1028,14 @@ int _cdecl wmain(int argc, __in_ecount(argc) LPWSTR argv[])
     // jscript9.dll etc with ch. Since that isn't a concern on non-Windows
     // builds, it's safe to conditionally compile it out.
 #ifdef _WIN32
-    ATOM lock = ::AddAtom(szChakraCoreLock);
+    lock = ::AddAtom(szChakraCoreLock);
     AssertMsg(lock, "failed to lock chakracore.dll");
 #endif // _WIN32
 
     HostConfigFlags::HandleArgsFlag(argc, argv);
 
-    ChakraRTInterface::ArgInfo argInfo = { argc, argv, PrintUsage, nullptr };
-    HINSTANCE chakraLibrary = nullptr;
-    bool success = ChakraRTInterface::LoadChakraDll(&argInfo, &chakraLibrary);
+    argInfo = { argc, argv, PrintUsage, nullptr };
+    success = ChakraRTInterface::LoadChakraDll(&argInfo, &chakraLibrary);
 
 #if defined(CHAKRA_STATIC_LIBRARY) && !defined(NDEBUG)
     // handle command line flags
@@ -975,7 +1047,6 @@ int _cdecl wmain(int argc, __in_ecount(argc) LPWSTR argv[])
         WideStringToNarrowDynamic(argv[1], &argInfo.filename);
     }
 
-    HRESULT exitCode = E_FAIL;
     if (success)
     {
 #ifdef _WIN32
@@ -1016,5 +1087,20 @@ int _cdecl wmain(int argc, __in_ecount(argc) LPWSTR argv[])
 #endif
 
     PAL_Shutdown();
-    return (int)exitCode;
+    retval = (int)exitCode;
+
+return_cleanup:
+#ifndef _WIN32
+    if(argv != nullptr)
+    {
+        for(int i=0;i<origargc;i++)
+        {
+            free(argv[i]);
+            argv[i] = nullptr;
+        }
+    }
+    delete[] argv;
+    argv = nullptr;
+#endif
+    return retval;
 }
