@@ -8,17 +8,20 @@
 (function (EngineInterface, InitType) {
     var platform = EngineInterface.Intl;
     if (platform.localeLookupCache === undefined) {
-        platform.localeLookupCache = new Map();
+        platform.localeLookupCache = new platform.Map();
     }
     if (platform.localeBestFitCache === undefined) {
-        platform.localeBestFitCache = new Map();
+        platform.localeBestFitCache = new platform.Map();
     }
 
+    // constants
+    const NOT_FOUND = "NOT_FOUND";
+    const CURRENCY_CODE_RE = /^[A-Z]{3}$/i;
+
     // Built-Ins
     var setPrototype = platform.builtInSetPrototype;
     var getArrayLength = platform.builtInGetArrayLength;
     var callInstanceFunc = platform.builtInCallInstanceFunction;
-    const NOT_FOUND = "NOT_FOUND";
 
     var Boolean = platform.Boolean;
     var Object = platform.Object;
@@ -27,6 +30,7 @@
     var String = platform.String;
     var Date = platform.Date;
     var Error = platform.Error;
+    var Map = platform.Map;
 
     var RaiseAssert = platform.raiseAssert;
 
@@ -37,16 +41,15 @@
         pow: platform.builtInMathPow
     }, null);
 
-    var objectDefineProperty = platform.builtInJavascriptObjectEntryDefineProperty;
     var ObjectGetPrototypeOf = platform.builtInJavascriptObjectEntryGetPrototypeOf;
     var ObjectIsExtensible = platform.builtInJavascriptObjectEntryIsExtensible;
     var ObjectGetOwnPropertyNames = platform.builtInJavascriptObjectEntryGetOwnPropertyNames;
     var ObjectInstanceHasOwnProperty = platform.builtInJavascriptObjectEntryHasOwnProperty;
-
     // Because we don't keep track of the attributes object, and neither does the internals of Object.defineProperty;
     // We don't need to restore it's prototype.
+    var _objectDefineProperty = platform.builtInJavascriptObjectEntryDefineProperty;
     var ObjectDefineProperty = function (obj, prop, attributes) {
-        objectDefineProperty(obj, prop, setPrototype(attributes, null));
+        _objectDefineProperty(obj, prop, setPrototype(attributes, null));
     };
 
     var ArrayInstanceForEach = platform.builtInJavascriptArrayEntryForEach;
@@ -67,6 +70,35 @@
     var isFinite = platform.builtInGlobalObjectEntryIsFinite;
     var isNaN = platform.builtInGlobalObjectEntryIsNaN;
 
+    let GetDefaultLocale = function () {
+        let defaultLocale = platform.getDefaultLocale();
+        if (defaultLocale === null) {
+            // TODO (doilij): remove this fallback when implemented under ICU
+            defaultLocale = "en-US";
+        }
+        return defaultLocale;
+    }
+
+    let CacheNumberFormat = function (numberFormat) {
+        let retVal = platform.cacheNumberFormat(numberFormat);
+        if (retVal === null) {
+            // TODO (doilij): remove this fallback when implemented under ICU
+            numberFormat.__numberingSystem = "";
+        }
+    }
+
+    let CreateDateTimeFormat = function (dateTimeFormat, condition) {
+        let retVal = platform.createDateTimeFormat(dateTimeFormat, condition);
+        if (retVal === null) {
+            // TODO (doilij): remove this fallback when implemented under ICU
+            dateTimeFormat.__numberingSystem = "";
+            dateTimeFormat.__patternStrings = [
+                "{month.a}{day.b}{hour.c}{minute.d}{second.e}",
+                "" // another entry for fun
+            ]
+        }
+    }
+
     var forEach = function (obj, length, func) {
         var current = 0;
 
@@ -91,6 +123,10 @@
         var resolvedLocale = platform.localeBestFitCache.get(locale);
         if (resolvedLocale === undefined) {
             resolvedLocale = platform.resolveLocaleBestFit(locale);
+            if (resolvedLocale === null) {
+                // TODO (doilij): remove this fallback when implemented under ICU
+                resolvedLocale = undefined;
+            }
             // If resolvedLocale is undefined, cache that we got undefined
             // so we don't try to resolve for `locale` in future.
             platform.localeBestFitCache.set(locale, resolvedLocale === undefined ? NOT_FOUND : resolvedLocale);
@@ -107,6 +143,10 @@
         var resolvedLocale = platform.localeLookupCache.get(localeWithoutSubtags);
         if (resolvedLocale === undefined) {
             resolvedLocale = platform.resolveLocaleLookup(localeWithoutSubtags);
+            if (resolvedLocale === null) {
+                // TODO (doilij): remove this fallback when implemented under ICU
+                resolvedLocale = undefined;
+            }
             // If resolvedLocale is undefined, cache that we got undefined
             // so we don't try to resolve for `locale` in future.
             platform.localeLookupCache.set(localeWithoutSubtags, resolvedLocale === undefined ? NOT_FOUND : resolvedLocale);
@@ -116,8 +156,18 @@
         return resolvedLocale;
     }
 
+    var getExtensionSubtags = function (locale) {
+        // TODO (doilij): implement this with BCP 47 Language-Tag RegExps
+        return [];
+    }
+
     var resolveLocaleHelper = function (locale, fitter, extensionFilter, defaultLocale) {
         var subTags = platform.getExtensions(locale);
+        if (subTags === null) {
+            // platform.getExtensions returns null to indicate fallback to JS implementation
+            subTags = getExtensionSubtags(locale);
+        }
+
         if (subTags) {
             callInstanceFunc(ArrayInstanceForEach, subTags, (function (subTag) { locale = callInstanceFunc(StringInstanceReplace, locale, "-" + subTag, ""); }));
         }
@@ -164,7 +214,7 @@
     }
 
     var strippedDefaultLocale = function () {
-        return platform.builtInRegexMatch(platform.getDefaultLocale(), /([^_]*).*/)[1];
+        return platform.builtInRegexMatch(GetDefaultLocale(), /([^_]*).*/)[1];
     };
 
     var Internal = (function () {
@@ -180,7 +230,6 @@
                 return s !== undefined ? String(s) : undefined;
             },
 
-
             ToNumber: function (n) {
                 return n === undefined ? NaN : Number(n);
             },
@@ -210,9 +259,6 @@
         }, null)
     })();
 
-    // Common Regexps
-    var currencyCodeRE = RegExp("^[A-Z]{3}$", "i");
-
     // Internal ops implemented in JS:
     function GetOption(options, property, type, values, fallback) {
         var value = options[property];
@@ -259,7 +305,7 @@
     function IsWellFormedCurrencyCode(code) {
         code = Internal.ToString(code);
 
-        return platform.builtInRegexMatch(code, currencyCodeRE) !== null;
+        return platform.builtInRegexMatch(code, CURRENCY_CODE_RE) !== null;
     }
 
     // Make sure locales is an array, remove duplicate locales, make sure each locale is valid, and canonicalize each.
@@ -273,17 +319,17 @@
         }
 
         locales = Internal.ToObject(locales);
-        var length = Internal.ToUint32(locales.length);
+        const length = Internal.ToUint32(locales.length);
 
         // TODO: Use sets here to prevent duplicates
-        var seen = [];
+        let seen = [];
 
-        forEach(locales, length, (function (locale) {
+        forEach(locales, length, function (locale) {
             if ((typeof locale !== 'string' && typeof locale !== 'object') || locale === null) {
                 platform.raiseNeedObjectOrString("Locale");
             }
 
-            var tag = Internal.ToString(locale);
+            let tag = Internal.ToString(locale);
 
             if (!platform.isWellFormedLanguageTag(tag)) {
                 platform.raiseLocaleNotWellFormed(String(tag));
@@ -294,8 +340,7 @@
             if (tag !== undefined && callInstanceFunc(ArrayInstanceIndexOf, seen, tag) === -1) {
                 callInstanceFunc(ArrayInstancePush, seen, tag);
             }
-
-        }));
+        });
 
         return seen;
     }
@@ -325,13 +370,26 @@
         }
 
         var hiddenObj = platform.getHiddenObject(that);
-        if (hiddenObj === undefined || hiddenObj.isValid !== "Valid") {
+        if (!hiddenObj || hiddenObj.isValid !== "Valid") {
             platform.raiseNotAConstructor(functionName);
         }
 
         return supportedLocalesOf(locales, options);
     }
 
+    var canonicalizeLocaleListWrapper = function (that, functionName, locales) {
+        if (that === null || that === undefined) {
+            platform.raiseNotAConstructor(functionName);
+        }
+
+        var hiddenObj = platform.getHiddenObject(that);
+        if (!hiddenObj || hiddenObj.isValid !== "Valid") {
+            platform.raiseNotAConstructor(functionName);
+        }
+
+        return CanonicalizeLocaleList(locales);
+    }
+
     // Shared among all the constructors
     var supportedLocalesOf = function (locales, options) {
         var matcher;
@@ -356,45 +414,46 @@
         }
     };
 
-    var supportedLocalesOfThisArg = setPrototype({}, null);
-    platform.setHiddenObject(supportedLocalesOfThisArg, setPrototype({ isValid: "Valid" }, null));
-
-    // Because we need to display correct error message for each Intl type, we have these helper functions
-    var collator_supportedLocalesOf_name = "Intl.Collator.supportedLocalesOf";
-    var collator_supportedLocalesOf = callInstanceFunc(FunctionInstanceBind, tagPublicFunction(collator_supportedLocalesOf_name, function (locales) {
-        var options = arguments.length < 2 ? undefined : arguments[1];
+    const intlStaticMethodThisArg = setPrototype({}, null);
+    platform.setHiddenObject(intlStaticMethodThisArg, setPrototype({ isValid: "Valid" }, null));
 
+    // We wrap these functions so that we can define the correct name for this function for each Intl constructor,
+    // which allows us to display the correct error message for each Intl type.
+    const collator_supportedLocalesOf_name = "Intl.Collator.supportedLocalesOf";
+    const collator_supportedLocalesOf = callInstanceFunc(FunctionInstanceBind, tagPublicFunction(collator_supportedLocalesOf_name,
+        function collator_supportedLocalesOf_dummyName(locales) {
+        const options = arguments.length < 2 ? undefined : arguments[1];
         return supportedLocalesOfWrapper(this, collator_supportedLocalesOf_name, locales, options);
-    }), supportedLocalesOfThisArg);
-    // TODO: Bound functions get the "bound" prefix by default, so we need to remove it.
-    // When https://github.com/Microsoft/ChakraCore/issues/637 is fixed and we have a way
-    // to make built-in functions non-constructible, we can remove the call to
-    // Function.prototype.bind and just rely on tagging instead of setting the "name" manually.
-    Object.defineProperty(collator_supportedLocalesOf, 'name', { value: 'supportedLocalesOf' });
-
-    var numberFormat_supportedLocalesOf_name = "Intl.NumberFormat.supportedLocalesOf";
-    var numberFormat_supportedLocalesOf = callInstanceFunc(FunctionInstanceBind, tagPublicFunction(numberFormat_supportedLocalesOf_name, function (locales) {
-        var options = arguments.length < 2 ? undefined : arguments[1];
+        }), intlStaticMethodThisArg);
 
+    const numberFormat_supportedLocalesOf_name = "Intl.NumberFormat.supportedLocalesOf";
+    const numberFormat_supportedLocalesOf = callInstanceFunc(FunctionInstanceBind, tagPublicFunction(numberFormat_supportedLocalesOf_name,
+        function numberFormat_supportedLocalesOf_dummyName(locales) {
+        const options = arguments.length < 2 ? undefined : arguments[1];
         return supportedLocalesOfWrapper(this, numberFormat_supportedLocalesOf_name, locales, options);
-    }), supportedLocalesOfThisArg);
-    // TODO: Bound functions get the "bound" prefix by default, so we need to remove it.
-    // When https://github.com/Microsoft/ChakraCore/issues/637 is fixed and we have a way
-    // to make built-in functions non-constructible, we can remove the call to
-    // Function.prototype.bind and just rely on tagging instead of setting the "name" manually.
-    Object.defineProperty(numberFormat_supportedLocalesOf, 'name', { value: 'supportedLocalesOf' });
-
-    var dateTimeFormat_supportedLocalesOf_name = "Intl.DateTimeFormat.supportedLocalesOf";
-    var dateTimeFormat_supportedLocalesOf = callInstanceFunc(FunctionInstanceBind, tagPublicFunction(dateTimeFormat_supportedLocalesOf_name, function (locales) {
-        var options = arguments.length < 2 ? undefined : arguments[1];
+        }), intlStaticMethodThisArg);
 
+    const dateTimeFormat_supportedLocalesOf_name = "Intl.DateTimeFormat.supportedLocalesOf";
+    const dateTimeFormat_supportedLocalesOf = callInstanceFunc(FunctionInstanceBind, tagPublicFunction(dateTimeFormat_supportedLocalesOf_name,
+        function dateTimeFormat_supportedLocalesOf_dummyName(locales) {
+        const options = arguments.length < 2 ? undefined : arguments[1];
         return supportedLocalesOfWrapper(this, dateTimeFormat_supportedLocalesOf_name, locales, options);
-    }), supportedLocalesOfThisArg);
+        }), intlStaticMethodThisArg);
+
+    const getCanonicalLocales_name = "Intl.getCanonicalLocales";
+    const getCanonicalLocales = callInstanceFunc(FunctionInstanceBind, tagPublicFunction(getCanonicalLocales_name,
+        function getCanonicalLocales_dummyName(locales) {
+            return canonicalizeLocaleListWrapper(this, getCanonicalLocales_name, locales);
+        }), intlStaticMethodThisArg);
+
     // TODO: Bound functions get the "bound" prefix by default, so we need to remove it.
     // When https://github.com/Microsoft/ChakraCore/issues/637 is fixed and we have a way
     // to make built-in functions non-constructible, we can remove the call to
-    // Function.prototype.bind and just rely on tagging instead of setting the "name" manually.
-    Object.defineProperty(dateTimeFormat_supportedLocalesOf, 'name', { value: 'supportedLocalesOf' });
+    // Function.prototype.bind (i.e. FunctionInstanceBind) and just rely on tagging instead of setting the "name" manually.
+    ObjectDefineProperty(collator_supportedLocalesOf, 'name', { value: 'supportedLocalesOf' });
+    ObjectDefineProperty(numberFormat_supportedLocalesOf, 'name', { value: 'supportedLocalesOf' });
+    ObjectDefineProperty(dateTimeFormat_supportedLocalesOf, 'name', { value: 'supportedLocalesOf' });
+    ObjectDefineProperty(getCanonicalLocales, 'name', { value: 'getCanonicalLocales' });
 
     // If an empty string is encountered for the value of the property; that means that is by default.
     // So in the case of zh-TW; "default" and "stroke" are the same.
@@ -437,7 +496,7 @@
     }());
 
     var mappedDefaultLocale = function () {
-        var parts = platform.builtInRegexMatch(platform.getDefaultLocale(), /([^_]*)_?(.+)?/);
+        var parts = platform.builtInRegexMatch(GetDefaultLocale(), /([^_]*)_?(.+)?/);
         var locale = parts[1];
         var collation = parts[2];
         var availableBcpTags = reverseLocaleAcceptingCollationValues[locale];
@@ -561,20 +620,20 @@
             }
 
             platform.registerBuiltInFunction(tagPublicFunction("String.prototype.localeCompare", function () {
-                    var that = arguments[0];
-                    if (this === undefined || this === null) {
-                        platform.raiseThis_NullOrUndefined("String.prototype.localeCompare");
-                    }
-                    else if (that === null) {
-                        platform.raiseNeedObject();
-                    }
-                    // ToString must be called on this/that argument before we do any other operation, as other operations in InitializeCollator may also be observable
-                    var thisArg = String(this);
-                    var that = String(that);
-                    var stateObject = setPrototype({}, null);
-                    InitializeCollator(stateObject, arguments[1], arguments[2]);
-                    return Number(platform.compareString(thisArg, that, stateObject.__localeForCompare, stateObject.__sensitivity, stateObject.__ignorePunctuation, stateObject.__numeric));
-                }), 4);
+                var that = arguments[0];
+                if (this === undefined || this === null) {
+                    platform.raiseThis_NullOrUndefined("String.prototype.localeCompare");
+                }
+                else if (that === null) {
+                    platform.raiseNeedObject();
+                }
+                // ToString must be called on this/that argument before we do any other operation, as other operations in InitializeCollator may also be observable
+                var thisArg = String(this);
+                var that = String(that);
+                var stateObject = setPrototype({}, null);
+                InitializeCollator(stateObject, arguments[1], arguments[2]);
+                return Number(platform.compareString(thisArg, that, stateObject.__localeForCompare, stateObject.__sensitivity, stateObject.__ignorePunctuation, stateObject.__numeric));
+            }), 4);
 
             if (InitType === 'Intl') {
 
@@ -612,8 +671,6 @@
                 }
                 tagPublicFunction("Intl.Collator", Collator);
 
-                tagPublicFunction("Intl.Collator.prototype.compare", compare);
-
                 function compare(a, b) {
                     if (typeof this !== 'object') {
                         platform.raiseNeedObjectOfType("Collator.prototype.compare", "Collator");
@@ -629,6 +686,7 @@
 
                     return Number(platform.compareString(a, b, hiddenObject.__localeForCompare, hiddenObject.__sensitivity, hiddenObject.__ignorePunctuation, hiddenObject.__numeric));
                 }
+                tagPublicFunction("Intl.Collator.prototype.compare", compare);
 
                 ObjectDefineProperty(Collator, 'supportedLocalesOf', { value: collator_supportedLocalesOf, writable: true, configurable: true });
 
@@ -683,11 +741,8 @@
 
     // Intl.NumberFormat, Number.prototype.toLocaleString
     var NumberFormat = (function () {
-
         if (InitType === 'Intl' || InitType === 'Number') {
-
             function InitializeNumberFormat(numberFormat, localeList, options) {
-
                 if (typeof numberFormat != "object") {
                     platform.raiseNeedObject();
                 }
@@ -787,7 +842,7 @@
 
                 try {
                     // Cache api instance and update numbering system on the object
-                    platform.cacheNumberFormat(numberFormat);
+                    CacheNumberFormat(numberFormat);
                 }
                 catch (e) {
                     throwExIfOOMOrSOE(e);
@@ -802,17 +857,17 @@
             }
 
             platform.registerBuiltInFunction(tagPublicFunction("Number.prototype.toLocaleString", function () {
-                    if ((typeof this) !== 'number' && !(this instanceof Number)) {
-                        platform.raiseNeedObjectOfType("Number.prototype.toLocaleString", "Number");
-                    }
+                if ((typeof this) !== 'number' && !(this instanceof Number)) {
+                    platform.raiseNeedObjectOfType("Number.prototype.toLocaleString", "Number");
+                }
 
-                    var stateObject = setPrototype({}, null);
-                    InitializeNumberFormat(stateObject, arguments[0], arguments[1]);
+                var stateObject = setPrototype({}, null);
+                InitializeNumberFormat(stateObject, arguments[0], arguments[1]);
 
-                    var n = Internal.ToNumber(this);
-                    // Need to special case the '-0' case to format as 0 instead of -0.
-                    return String(platform.formatNumber(n === -0 ? 0 : n, stateObject));
-                }), 3);
+                var n = Internal.ToNumber(this);
+                // Need to special case the '-0' case to format as 0 instead of -0.
+                return String(platform.formatNumber(n === -0 ? 0 : n, stateObject));
+            }), 3);
 
             if (InitType === 'Intl') {
                 function NumberFormat() {
@@ -933,12 +988,14 @@
 
                 var needDefaults = true;
                 if (required === "date" || required === "any") {
-                    if (options.weekday !== undefined || options.year !== undefined || options.month !== undefined || options.day !== undefined)
+                    if (options.weekday !== undefined || options.year !== undefined || options.month !== undefined || options.day !== undefined) {
                         needDefaults = false;
+                    }
                 }
                 if (required === "time" || required === "any") {
-                    if (options.hour !== undefined || options.minute !== undefined || options.second !== undefined)
+                    if (options.hour !== undefined || options.minute !== undefined || options.second !== undefined) {
                         needDefaults = false;
+                    }
                 }
 
                 if (needDefaults && (defaults === "date" || defaults === "all")) {
@@ -1047,30 +1104,33 @@
 
             // Certain formats have similar patterns on both ecma and windows; will use helper methods for them
             function correctWeekdayEraMonthPattern(patternString, userValue, searchParam) {
-                // parts[1] is either dayofweek.solo, dayofweek, era or month; parts[3] is either abbreviated or full
-                var parts = platform.builtInRegexMatch(patternString, RegExp("{(" + searchParam + "(\\.solo)?)\\.([a-z]*)(\\([0-9]\\))?}"));
+                // parts[1] is either dayofweek.solo, dayofweek, era or month; parts[2] is either abbreviated or full
+                var parts = platform.builtInRegexMatch(patternString, RegExp("{(" + searchParam + "(?:\\.solo)?)\\.([a-z]*)(?:\\([0-9]\\))?}"));
                 // If this happens that means windows removed the specific pattern (which isn't expected; but better be safe)
                 if (parts === null) {
                     RaiseAssert(new Error("Error when correcting windows returned weekday/Era/Month pattern; regex returned null. \nInput was: '" + patternString + "'\nRegex: '" + "{(" + searchParam + "(\\.solo)?)\\.([a-z]*)(\\([0-9]\\))?}'"));
                     return patternString;
                 }
 
-                if (parts[3] !== "full" && userValue === "long") {
+                if (parts[2] !== "full" && userValue === "long") {
                     return callInstanceFunc(StringInstanceReplace, patternString, parts[0], "{" + parts[1] + "." + "full" + "}");
                 } else if (userValue !== "long") {
                     return callInstanceFunc(StringInstanceReplace, patternString, parts[0], "{" + parts[1] + "." + (userValue === "short" ? "abbreviated" : "abbreviated(1)") + "}");
                 }
                 return patternString;
             }
+
             function correctDayHourMinuteSecondMonthPattern(patternString, userValue, searchParam) {
                 // parts[1] is either month, day, hour, minute, or second
-                var parts = platform.builtInRegexMatch(patternString, RegExp("{(" + searchParam + ")(?:\\.solo)?\\.([a-z]*)(\\([0-9]\\))?}"));
+                // REVIEW (doilij) is it even possible to have a '.solo' (i.e. /(?:\\.solo)?/ ) in the above cases?
+                var parts = platform.builtInRegexMatch(patternString, RegExp("{(" + searchParam + ")(?:\\.solo)?\\.([a-z]*)(?:\\([0-9]\\))?}"));
                 if (parts === null) {
                     RaiseAssert(new Error("Error when correcting windows returned day/hour/minute/second/month pattern; regex returned null. \nInput was: '" + patternString + "'\nRegex: '" + "{(" + searchParam + "(\\.solo)?)\\.([a-z]*)(\\([0-9]\\))?}'"));
                     return patternString;
                 }
 
-                if (userValue === "2-digit") { // Only correct the 2 digit; unless part[2] isn't integer
+                // Only correct the 2 digit; unless part[2] isn't integer
+                if (userValue === "2-digit") {
                     return callInstanceFunc(StringInstanceReplace, patternString, parts[0], "{" + parts[1] + ".integer(2)}");
                 } else if (parts[2] !== "integer") {
                     return callInstanceFunc(StringInstanceReplace, patternString, parts[0], "{" + parts[1] + ".integer}");
@@ -1081,7 +1141,6 @@
 
             // Perhaps the level of validation that we have might not be required for this method
             function updatePatternStrings(patternString, dateTimeFormat) {
-
                 if (dateTimeFormat.__weekday !== undefined) {
                     patternString = correctWeekdayEraMonthPattern(patternString, dateTimeFormat.__weekday, "dayofweek");
                 }
@@ -1154,11 +1213,13 @@
 
                 var matcher = GetOption(options, "localeMatcher", "string", ["lookup", "best fit"], "best fit");
                 var timeZone = GetOption(options, "timeZone", "string", undefined, undefined);
+
                 if (timeZone !== undefined) {
                     timeZone = platform.validateAndCanonicalizeTimeZone(timeZone);
                 } else {
                     timeZone = platform.getDefaultTimeZone();
                 }
+
                 if (timeZone === undefined) {
                     platform.raiseOptionValueOutOfRange();
                 }
@@ -1229,7 +1290,7 @@
 
                 try {
                     // Create the DateTimeFormatter to extract pattern strings
-                    platform.createDateTimeFormat(dateTimeFormat, false);
+                    CreateDateTimeFormat(dateTimeFormat, false);
                 } catch (e) {
                     // Rethrow SOE or OOM
                     throwExIfOOMOrSOE(e);
@@ -1240,6 +1301,7 @@
                     dateTimeFormat.__templateString = EcmaOptionsToWindowsTemplate(defaultOptions, null);
                     errorThrown = true;
                 }
+
                 if (!errorThrown) {
                     // Update the pattern strings
                     dateTimeFormat.__templateString = updatePatternStrings(dateTimeFormat.__patternStrings[0], dateTimeFormat);
@@ -1247,7 +1309,7 @@
 
                 try {
                     // Cache the date time formatter
-                    platform.createDateTimeFormat(dateTimeFormat, true);
+                    CreateDateTimeFormat(dateTimeFormat, true);
                 } catch (e) {
                     // Rethrow SOE or OOM
                     throwExIfOOMOrSOE(e);
@@ -1268,7 +1330,6 @@
                 dateTimeFormat.__initializedDateTimeFormat = true;
             }
 
-            
             platform.registerBuiltInFunction(tagPublicFunction("Date.prototype.toLocaleString", function () {
                 if (typeof this !== 'object' || !(this instanceof Date)) {
                     platform.raiseNeedObjectOfType("Date.prototype.toLocaleString", "Date");
@@ -1307,6 +1368,7 @@
                 InitializeDateTimeFormat(stateObject, arguments[0], ToDateTimeOptions(arguments[1], "time", "time"));
                 return String(platform.formatDateTime(Internal.ToNumber(this), stateObject));
             }), 2);
+
             if (InitType === 'Intl') {
                 function DateTimeFormat() {
                     // The function should have length of 0, hence we are getting arguments this way.
@@ -1341,7 +1403,6 @@
                 tagPublicFunction("Intl.DateTimeFormat", DateTimeFormat);
 
                 function format() {
-
                     if (typeof this !== 'object') {
                         platform.raiseNeedObjectOfType("DateTimeFormat.prototype.format", "DateTimeFormat");
                     }
@@ -1417,8 +1478,6 @@
 
                 ObjectDefineProperty(DateTimeFormat, 'supportedLocalesOf', { value: dateTimeFormat_supportedLocalesOf, writable: true, configurable: true });
 
-
-
                 return DateTimeFormat;
             }
         }
@@ -1431,5 +1490,6 @@
         ObjectDefineProperty(Intl, "Collator", { value: Collator, writable: true, enumerable: false, configurable: true });
         ObjectDefineProperty(Intl, "NumberFormat", { value: NumberFormat, writable: true, enumerable: false, configurable: true });
         ObjectDefineProperty(Intl, "DateTimeFormat", { value: DateTimeFormat, writable: true, enumerable: false, configurable: true });
+        ObjectDefineProperty(Intl, "getCanonicalLocales",   { value: getCanonicalLocales,   writable: true, enumerable: false, configurable: true });
     }
 });
