@@ -12,6 +12,8 @@
 #define PreInitializedObjectTypeCount ((MaxPreInitializedObjectTypeInlineSlotCount / InlineSlotCountIncrement) + 1)
 CompileAssert(MaxPreInitializedObjectTypeInlineSlotCount <= USHRT_MAX);
 
+#include "StringCache.h"
+
 class ScriptSite;
 class ActiveScriptExternalLibrary;
 class ProjectionExternalLibrary;
@@ -43,7 +45,7 @@ namespace Js
     typedef JsUtil::BaseDictionary<JavascriptMethod, JavascriptFunction*, Recycler, PowerOf2SizePolicy> BuiltInLibraryFunctionMap;
 
     // valid if object!= NULL
-    struct EnumeratedObjectCache 
+    struct EnumeratedObjectCache
     {
         static const int kMaxCachedPropStrings = 16;
         Field(DynamicObject*) object;
@@ -52,8 +54,20 @@ namespace Js
         Field(int) validPropStrings;
     };
 
+    struct PropertyStringMap
+    {
+        Field(PropertyString*) strLen2[80];
+
+        inline static uint PStrMapIndex(char16 ch)
+        {
+            Assert(ch >= '0' && ch <= 'z');
+            return ch - '0';
+        }
+    };
+
     struct Cache
     {
+        Field(PropertyStringMap*) propertyStrings[80];
         Field(JavascriptString *) lastNumberToStringRadix10String;
         Field(EnumeratedObjectCache) enumObjCache;
         Field(JavascriptString *) lastUtcTimeFromStrString;
@@ -194,7 +208,6 @@ namespace Js
         static DWORD GetBooleanFalseOffset() { return offsetof(JavascriptLibrary, booleanFalse); }
         static DWORD GetNegativeZeroOffset() { return offsetof(JavascriptLibrary, negativeZero); }
         static DWORD GetNumberTypeStaticOffset() { return offsetof(JavascriptLibrary, numberTypeStatic); }
-        static DWORD GetStringTypeStaticOffset() { return offsetof(JavascriptLibrary, stringTypeStatic); }
         static DWORD GetObjectTypesOffset() { return offsetof(JavascriptLibrary, objectTypes); }
         static DWORD GetObjectHeaderInlinedTypesOffset() { return offsetof(JavascriptLibrary, objectHeaderInlinedTypes); }
         static DWORD GetRegexTypeOffset() { return offsetof(JavascriptLibrary, regexType); }
@@ -244,7 +257,6 @@ namespace Js
         Field(DynamicType *) arrayBufferType;
         Field(DynamicType *) sharedArrayBufferType;
         Field(DynamicType *) dataViewType;
-        Field(DynamicType *) typedArrayType;
         Field(DynamicType *) int8ArrayType;
         Field(DynamicType *) uint8ArrayType;
         Field(DynamicType *) uint8ClampedArrayType;
@@ -317,6 +329,7 @@ namespace Js
         Field(DynamicType *) webAssemblyMemoryType;
         Field(DynamicType *) webAssemblyTableType;
 
+#ifdef ENABLE_SIMDJS
         // SIMD_JS
         Field(DynamicType *) simdBool8x16TypeDynamic;
         Field(DynamicType *) simdBool16x8TypeDynamic;
@@ -340,6 +353,7 @@ namespace Js
         Field(StaticType *) simdUint32x4TypeStatic;
         Field(StaticType *) simdUint16x8TypeStatic;
         Field(StaticType *) simdUint8x16TypeStatic;
+#endif // #ifdef ENABLE_SIMDJS
 
         Field(DynamicType *) numberTypeDynamic;
         Field(DynamicType *) objectTypes[PreInitializedObjectTypeCount];
@@ -347,7 +361,6 @@ namespace Js
         Field(DynamicType *) regexPrototypeType;
         Field(DynamicType *) regexType;
         Field(DynamicType *) regexResultType;
-        Field(StaticType  *) stringTypeStatic;
         Field(DynamicType *) stringTypeDynamic;
         Field(DynamicType *) mapType;
         Field(DynamicType *) setType;
@@ -359,59 +372,19 @@ namespace Js
         Field(DynamicType *) moduleNamespaceType;
         Field(PropertyDescriptor) defaultPropertyDescriptor;
 
+        Field(StringCache) stringCache; // cache string variables once they are used
+
+        // no late caching is needed
         Field(JavascriptString*) nullString;
         Field(JavascriptString*) emptyString;
-        Field(JavascriptString*) quotesString;
-        Field(JavascriptString*) whackString;
-        Field(JavascriptString*) objectDisplayString;
-        Field(JavascriptString*) stringTypeDisplayString;
-        Field(JavascriptString*) errorDisplayString;
-        Field(JavascriptString*) functionPrefixString;
-        Field(JavascriptString*) generatorFunctionPrefixString;
-        Field(JavascriptString*) asyncFunctionPrefixString;
-        Field(JavascriptString*) functionDisplayString;
-        Field(JavascriptString*) xDomainFunctionDisplayString;
-        Field(JavascriptString*) undefinedDisplayString;
-        Field(JavascriptString*) nanDisplayString;
-        Field(JavascriptString*) nullDisplayString;
-        Field(JavascriptString*) unknownDisplayString;
-        Field(JavascriptString*) commaDisplayString;
-        Field(JavascriptString*) commaSpaceDisplayString;
-        Field(JavascriptString*) trueDisplayString;
-        Field(JavascriptString*) falseDisplayString;
-        Field(JavascriptString*) lengthDisplayString;
-        Field(JavascriptString*) invalidDateString;
-        Field(JavascriptString*) objectTypeDisplayString;
-        Field(JavascriptString*) functionTypeDisplayString;
-        Field(JavascriptString*) booleanTypeDisplayString;
-        Field(JavascriptString*) numberTypeDisplayString;
-        Field(JavascriptString*) moduleTypeDisplayString;
-        Field(JavascriptString*) variantDateTypeDisplayString;
-
-        // SIMD_JS
-        Field(JavascriptString*) simdFloat32x4DisplayString;
-        Field(JavascriptString*) simdFloat64x2DisplayString;
-        Field(JavascriptString*) simdInt32x4DisplayString;
-        Field(JavascriptString*) simdInt16x8DisplayString;
-        Field(JavascriptString*) simdInt8x16DisplayString;
-        Field(JavascriptString*) simdBool32x4DisplayString;
-        Field(JavascriptString*) simdBool16x8DisplayString;
-        Field(JavascriptString*) simdBool8x16DisplayString;
-        Field(JavascriptString*) simdUint32x4DisplayString;
-        Field(JavascriptString*) simdUint16x8DisplayString;
-        Field(JavascriptString*) simdUint8x16DisplayString;
-
-
 
+        // used implicitly by ChakraFull - don't late cache
         Field(JavascriptString*) symbolTypeDisplayString;
         Field(JavascriptString*) debuggerDeadZoneBlockVariableString;
 
         Field(DynamicObject*) missingPropertyHolder;
-
         Field(StaticType*) throwErrorObjectType;
-
         Field(PropertyStringCacheMap*) propertyStringMap;
-
         Field(ConstructorCache*) builtInConstructorCache;
 
 #ifdef ENABLE_DEBUG_CONFIG_OPTIONS
@@ -420,7 +393,6 @@ namespace Js
 #endif
 
         Field(JavascriptFunction*) evalFunctionObject;
-        Field(JavascriptFunction*) arrayPrototypeValuesFunction;
         Field(JavascriptFunction*) parseIntFunctionObject;
         Field(JavascriptFunction*) parseFloatFunctionObject;
         Field(JavascriptFunction*) arrayPrototypeToStringFunction;
@@ -435,8 +407,12 @@ namespace Js
 
 #ifdef ENABLE_WASM
         Field(DynamicObject*) webAssemblyObject;
+        Field(JavascriptFunction*) webAssemblyQueryResponseFunction;
+        Field(JavascriptFunction*) webAssemblyCompileFunction;
+        Field(JavascriptFunction*) webAssemblyInstantiateBoundFunction;
 #endif
 
+#ifdef ENABLE_SIMDJS
         // SIMD_JS
         Field(JavascriptFunction*) simdFloat32x4ToStringFunction;
         Field(JavascriptFunction*) simdFloat64x2ToStringFunction;
@@ -449,8 +425,7 @@ namespace Js
         Field(JavascriptFunction*) simdUint32x4ToStringFunction;
         Field(JavascriptFunction*) simdUint16x8ToStringFunction;
         Field(JavascriptFunction*) simdUint8x16ToStringFunction;
-
-
+#endif
 
         Field(JavascriptSymbol*) symbolMatch;
         Field(JavascriptSymbol*) symbolReplace;
@@ -549,6 +524,7 @@ namespace Js
         void AddAccessorsToLibraryObject(DynamicObject* object, PropertyId propertyId, FunctionInfo * getterFunctionInfo, FunctionInfo * setterFunctionInfo);
         void AddAccessorsToLibraryObject(DynamicObject* object, PropertyId propertyId, RecyclableObject * getterFunction, RecyclableObject * setterFunction);
         void AddAccessorsToLibraryObjectWithName(DynamicObject* object, PropertyId propertyId, PropertyId nameId, FunctionInfo * getterFunctionInfo, FunctionInfo * setterFunction);
+        void AddSpeciesAccessorsToLibraryObject(DynamicObject* object, FunctionInfo * getterFunctionInfo);
         RuntimeFunction * CreateGetterFunction(PropertyId nameId, FunctionInfo* functionInfo);
         RuntimeFunction * CreateSetterFunction(PropertyId nameId, FunctionInfo* functionInfo);
 
@@ -562,14 +538,14 @@ namespace Js
         static SimpleTypeHandler<1> SharedFunctionWithLengthTypeHandler;
         static SimpleTypeHandler<2> SharedFunctionWithLengthAndNameTypeHandler;
         static SimpleTypeHandler<1> SharedIdMappedFunctionWithPrototypeTypeHandler;
-        static SimpleTypeHandler<2> SharedNamespaceSymbolTypeHandler;
+        static SimpleTypeHandler<1> SharedNamespaceSymbolTypeHandler;
         static MissingPropertyTypeHandler MissingPropertyHolderTypeHandler;
 
         static SimplePropertyDescriptor const SharedFunctionPropertyDescriptors[2];
         static SimplePropertyDescriptor const HeapArgumentsPropertyDescriptors[3];
         static SimplePropertyDescriptor const FunctionWithLengthAndPrototypeTypeDescriptors[2];
         static SimplePropertyDescriptor const FunctionWithLengthAndNameTypeDescriptors[2];
-        static SimplePropertyDescriptor const ModuleNamespaceTypeDescriptors[2];
+        static SimplePropertyDescriptor const ModuleNamespaceTypeDescriptors[1];
 
     public:
 
@@ -623,53 +599,76 @@ namespace Js
         JavascriptSymbol* GetSymbolSpecies() { return symbolSpecies; }
         JavascriptString* GetNullString() { return nullString; }
         JavascriptString* GetEmptyString() const;
-        JavascriptString* GetWhackString() { return whackString; }
-        JavascriptString* GetUndefinedDisplayString() { return undefinedDisplayString; }
-        JavascriptString* GetNaNDisplayString() { return nanDisplayString; }
-        JavascriptString* GetQuotesString() { return quotesString; }
-        JavascriptString* GetNullDisplayString() { return nullDisplayString; }
-        JavascriptString* GetUnknownDisplayString() { return unknownDisplayString; }
-        JavascriptString* GetCommaDisplayString() { return commaDisplayString; }
-        JavascriptString* GetCommaSpaceDisplayString() { return commaSpaceDisplayString; }
-        JavascriptString* GetTrueDisplayString() { return trueDisplayString; }
-        JavascriptString* GetFalseDisplayString() { return falseDisplayString; }
-        JavascriptString* GetLengthDisplayString() { return lengthDisplayString; }
-        JavascriptString* GetObjectDisplayString() { return objectDisplayString; }
-        JavascriptString* GetStringTypeDisplayString() { return stringTypeDisplayString; }
-        JavascriptString* GetErrorDisplayString() const { return errorDisplayString; }
-        JavascriptString* GetFunctionPrefixString() { return functionPrefixString; }
-        JavascriptString* GetGeneratorFunctionPrefixString() { return generatorFunctionPrefixString; }
-        JavascriptString* GetAsyncFunctionPrefixString() { return asyncFunctionPrefixString; }
-        JavascriptString* GetFunctionDisplayString() { return functionDisplayString; }
-        JavascriptString* GetXDomainFunctionDisplayString() { return xDomainFunctionDisplayString; }
-        JavascriptString* GetInvalidDateString() { return invalidDateString; }
-        JavascriptString* GetObjectTypeDisplayString() const { return objectTypeDisplayString; }
-        JavascriptString* GetFunctionTypeDisplayString() const { return functionTypeDisplayString; }
-        JavascriptString* GetBooleanTypeDisplayString() const { return booleanTypeDisplayString; }
-        JavascriptString* GetNumberTypeDisplayString() const { return numberTypeDisplayString; }
-        JavascriptString* GetModuleTypeDisplayString() const { return moduleTypeDisplayString; }
-        JavascriptString* GetVariantDateTypeDisplayString() const { return variantDateTypeDisplayString; }
 
-        // SIMD_JS
-        JavascriptString* GetSIMDFloat32x4DisplayString() const { return simdFloat32x4DisplayString; }
-        JavascriptString* GetSIMDFloat64x2DisplayString() const { return simdFloat64x2DisplayString; }
-        JavascriptString* GetSIMDInt32x4DisplayString()   const { return simdInt32x4DisplayString; }
-        JavascriptString* GetSIMDInt16x8DisplayString()   const { return simdInt16x8DisplayString; }
-        JavascriptString* GetSIMDInt8x16DisplayString()   const { return simdInt8x16DisplayString; }
-
-        JavascriptString* GetSIMDBool32x4DisplayString()   const { return simdBool32x4DisplayString; }
-        JavascriptString* GetSIMDBool16x8DisplayString()   const { return simdBool16x8DisplayString; }
-        JavascriptString* GetSIMDBool8x16DisplayString()   const { return simdBool8x16DisplayString; }
-
-        JavascriptString* GetSIMDUint32x4DisplayString()   const { return simdUint32x4DisplayString; }
-        JavascriptString* GetSIMDUint16x8DisplayString()   const { return simdUint16x8DisplayString; }
-        JavascriptString* GetSIMDUint8x16DisplayString()   const { return simdUint8x16DisplayString; }
+#define SCACHE_FUNCTION_PROXY(name) JavascriptString* name() { return stringCache.##name##(); }
+#ifdef ENABLE_SIMDJS
+        SCACHE_FUNCTION_PROXY(GetSIMDFloat32x4DisplayString)
+        SCACHE_FUNCTION_PROXY(GetSIMDFloat64x2DisplayString)
+        SCACHE_FUNCTION_PROXY(GetSIMDInt32x4DisplayString)
+        SCACHE_FUNCTION_PROXY(GetSIMDInt16x8DisplayString)
+        SCACHE_FUNCTION_PROXY(GetSIMDInt8x16DisplayString)
+        SCACHE_FUNCTION_PROXY(GetSIMDBool32x4DisplayString)
+        SCACHE_FUNCTION_PROXY(GetSIMDBool16x8DisplayString)
+        SCACHE_FUNCTION_PROXY(GetSIMDBool8x16DisplayString)
+        SCACHE_FUNCTION_PROXY(GetSIMDUint32x4DisplayString)
+        SCACHE_FUNCTION_PROXY(GetSIMDUint16x8DisplayString)
+        SCACHE_FUNCTION_PROXY(GetSIMDUint8x16DisplayString)
+#endif
+        SCACHE_FUNCTION_PROXY(GetEmptyObjectString)
+        SCACHE_FUNCTION_PROXY(GetQuotesString)
+        SCACHE_FUNCTION_PROXY(GetWhackString)
+        SCACHE_FUNCTION_PROXY(GetCommaDisplayString)
+        SCACHE_FUNCTION_PROXY(GetCommaSpaceDisplayString)
+        SCACHE_FUNCTION_PROXY(GetOpenBracketString)
+        SCACHE_FUNCTION_PROXY(GetCloseBracketString)
+        SCACHE_FUNCTION_PROXY(GetOpenSBracketString)
+        SCACHE_FUNCTION_PROXY(GetCloseSBracketString)
+        SCACHE_FUNCTION_PROXY(GetEmptyArrayString)
+        SCACHE_FUNCTION_PROXY(GetNewLineString)
+        SCACHE_FUNCTION_PROXY(GetColonString)
+        SCACHE_FUNCTION_PROXY(GetFunctionAnonymousString)
+        SCACHE_FUNCTION_PROXY(GetFunctionPTRAnonymousString)
+        SCACHE_FUNCTION_PROXY(GetAsyncFunctionAnonymouseString)
+        SCACHE_FUNCTION_PROXY(GetOpenRBracketString)
+        SCACHE_FUNCTION_PROXY(GetNewLineCloseRBracketString)
+        SCACHE_FUNCTION_PROXY(GetSpaceOpenBracketString)
+        SCACHE_FUNCTION_PROXY(GetNewLineCloseBracketString)
+        SCACHE_FUNCTION_PROXY(GetFunctionPrefixString)
+        SCACHE_FUNCTION_PROXY(GetGeneratorFunctionPrefixString)
+        SCACHE_FUNCTION_PROXY(GetAsyncFunctionPrefixString)
+        SCACHE_FUNCTION_PROXY(GetFunctionDisplayString)
+        SCACHE_FUNCTION_PROXY(GetXDomainFunctionDisplayString)
+        SCACHE_FUNCTION_PROXY(GetInvalidDateString)
+        SCACHE_FUNCTION_PROXY(GetObjectDisplayString)
+        SCACHE_FUNCTION_PROXY(GetObjectArgumentsDisplayString)
+        SCACHE_FUNCTION_PROXY(GetObjectArrayDisplayString)
+        SCACHE_FUNCTION_PROXY(GetObjectBooleanDisplayString)
+        SCACHE_FUNCTION_PROXY(GetObjectDateDisplayString)
+        SCACHE_FUNCTION_PROXY(GetObjectErrorDisplayString)
+        SCACHE_FUNCTION_PROXY(GetObjectFunctionDisplayString)
+        SCACHE_FUNCTION_PROXY(GetObjectNumberDisplayString)
+        SCACHE_FUNCTION_PROXY(GetObjectRegExpDisplayString)
+        SCACHE_FUNCTION_PROXY(GetObjectStringDisplayString)
+        SCACHE_FUNCTION_PROXY(GetUndefinedDisplayString)
+        SCACHE_FUNCTION_PROXY(GetNaNDisplayString)
+        SCACHE_FUNCTION_PROXY(GetNullDisplayString)
+        SCACHE_FUNCTION_PROXY(GetUnknownDisplayString)
+        SCACHE_FUNCTION_PROXY(GetTrueDisplayString)
+        SCACHE_FUNCTION_PROXY(GetFalseDisplayString)
+        SCACHE_FUNCTION_PROXY(GetStringTypeDisplayString)
+        SCACHE_FUNCTION_PROXY(GetObjectTypeDisplayString)
+        SCACHE_FUNCTION_PROXY(GetFunctionTypeDisplayString)
+        SCACHE_FUNCTION_PROXY(GetBooleanTypeDisplayString)
+        SCACHE_FUNCTION_PROXY(GetNumberTypeDisplayString)
+        SCACHE_FUNCTION_PROXY(GetModuleTypeDisplayString)
+        SCACHE_FUNCTION_PROXY(GetVariantDateTypeDisplayString)
+        SCACHE_FUNCTION_PROXY(GetSymbolTypeDisplayString)
+#undef  SCACHE_FUNCTION_PROXY
 
         JavascriptString* GetSymbolTypeDisplayString() const { return symbolTypeDisplayString; }
         JavascriptString* GetDebuggerDeadZoneBlockVariableString() { Assert(debuggerDeadZoneBlockVariableString); return debuggerDeadZoneBlockVariableString; }
         JavascriptRegExp* CreateEmptyRegExp();
         JavascriptFunction* GetEvalFunctionObject() { return evalFunctionObject; }
-        JavascriptFunction* GetArrayPrototypeValuesFunction() { return EnsureArrayPrototypeValuesFunction(); }
         JavascriptFunction* GetArrayIteratorPrototypeBuiltinNextFunction() { return arrayIteratorPrototypeBuiltinNextFunction; }
         DynamicObject* GetReflectObject() const { return reflectObject; }
         const PropertyDescriptor* GetDefaultPropertyDescriptor() const { return &defaultPropertyDescriptor; }
@@ -775,7 +774,13 @@ namespace Js
         DynamicType * GetWebAssemblyInstanceType()  const { return webAssemblyInstanceType; }
         DynamicType * GetWebAssemblyMemoryType() const { return webAssemblyMemoryType; }
         DynamicType * GetWebAssemblyTableType() const { return webAssemblyTableType; }
+#ifdef ENABLE_WASM
+        JavascriptFunction* GetWebAssemblyQueryResponseFunction() const { return webAssemblyQueryResponseFunction; }
+        JavascriptFunction* GetWebAssemblyCompileFunction() const { return webAssemblyCompileFunction; }
+        JavascriptFunction* GetWebAssemblyInstantiateBoundFunction() const { return webAssemblyInstantiateBoundFunction; }
+#endif
 
+#ifdef ENABLE_SIMDJS
         // SIMD_JS
         DynamicType * GetSIMDBool8x16TypeDynamic()  const { return simdBool8x16TypeDynamic;  }
         DynamicType * GetSIMDBool16x8TypeDynamic()  const { return simdBool16x8TypeDynamic;  }
@@ -799,6 +804,7 @@ namespace Js
         StaticType* GetSIMDUInt32x4TypeStatic()   const { return simdUint32x4TypeStatic; }
         StaticType* GetSIMDUint16x8TypeStatic()   const { return simdUint16x8TypeStatic; }
         StaticType* GetSIMDUint8x16TypeStatic()   const { return simdUint8x16TypeStatic; }
+#endif // #ifdef ENABLE_SIMDJS
 
         DynamicType * GetObjectLiteralType(uint16 requestedInlineSlotCapacity);
         DynamicType * GetObjectHeaderInlinedLiteralType(uint16 requestedInlineSlotCapacity);
@@ -820,7 +826,7 @@ namespace Js
         DynamicType * GetRegexType() const { return regexType; }
         DynamicType * GetRegexResultType() const { return regexResultType; }
         DynamicType * GetArrayBufferType() const { return arrayBufferType; }
-        StaticType  * GetStringTypeStatic() const { AssertMsg(stringTypeStatic, "Where's stringTypeStatic?"); return stringTypeStatic; }
+        StaticType  * GetStringTypeStatic() const { return stringCache.GetStringTypeStatic(); }
         DynamicType * GetStringTypeDynamic() const { return stringTypeDynamic; }
         StaticType  * GetVariantDateType() const { return variantDateType; }
         void EnsureDebugObject(DynamicObject* newDebugObject);
@@ -843,6 +849,7 @@ namespace Js
         JavascriptFunction* GetObjectValueOfFunction() const { return objectValueOfFunction; }
         JavascriptFunction* GetObjectToStringFunction() const { return objectToStringFunction; }
 
+#ifdef ENABLE_SIMDJS
         // SIMD_JS
         JavascriptFunction* GetSIMDFloat32x4ToStringFunction() const { return simdFloat32x4ToStringFunction;  }
         JavascriptFunction* GetSIMDFloat64x2ToStringFunction() const { return simdFloat64x2ToStringFunction; }
@@ -855,6 +862,7 @@ namespace Js
         JavascriptFunction* GetSIMDUint32x4ToStringFunction()   const { return simdUint32x4ToStringFunction; }
         JavascriptFunction* GetSIMDUint16x8ToStringFunction()   const { return simdUint16x8ToStringFunction; }
         JavascriptFunction* GetSIMDUint8x16ToStringFunction()   const { return simdUint8x16ToStringFunction; }
+#endif
 
         JavascriptFunction* GetDebugObjectNonUserGetterFunction() const { return debugObjectNonUserGetterFunction; }
         JavascriptFunction* GetDebugObjectNonUserSetterFunction() const { return debugObjectNonUserSetterFunction; }
@@ -917,8 +925,8 @@ namespace Js
         JavascriptArray* CreateArray(uint32 length, uint32 size);
         ArrayBuffer* CreateArrayBuffer(uint32 length);
         ArrayBuffer* CreateArrayBuffer(byte* buffer, uint32 length);
-        ArrayBuffer* CreateWebAssemblyArrayBuffer(uint32 length);
-        ArrayBuffer* CreateWebAssemblyArrayBuffer(byte* buffer, uint32 length);
+        class WebAssemblyArrayBuffer* CreateWebAssemblyArrayBuffer(uint32 length);
+        class WebAssemblyArrayBuffer* CreateWebAssemblyArrayBuffer(byte* buffer, uint32 length);
         SharedArrayBuffer* CreateSharedArrayBuffer(uint32 length);
         SharedArrayBuffer* CreateSharedArrayBuffer(SharedContents *contents);
         ArrayBuffer* CreateProjectionArraybuffer(uint32 length);
@@ -1034,7 +1042,7 @@ namespace Js
         JavascriptExternalFunction* CreateExternalFunction(ExternalMethod entryPointer, PropertyId nameId, Var signature, JavascriptTypeId prototypeTypeId, UINT64 flags);
         JavascriptExternalFunction* CreateExternalFunction(ExternalMethod entryPointer, Var nameId, Var signature, JavascriptTypeId prototypeTypeId, UINT64 flags);
         JavascriptExternalFunction* CreateStdCallExternalFunction(StdCallJavascriptMethod entryPointer, PropertyId nameId, void *callbackState);
-        JavascriptExternalFunction* CreateStdCallExternalFunction(StdCallJavascriptMethod entryPointer, Var nameId, void *callbackState);
+        JavascriptExternalFunction* CreateStdCallExternalFunction(StdCallJavascriptMethod entryPointer, Var name, void *callbackState);
         JavascriptPromiseAsyncSpawnExecutorFunction* CreatePromiseAsyncSpawnExecutorFunction(JavascriptMethod entryPoint, JavascriptGenerator* generator, Var target);
         JavascriptPromiseAsyncSpawnStepArgumentExecutorFunction* CreatePromiseAsyncSpawnStepArgumentExecutorFunction(JavascriptMethod entryPoint, JavascriptGenerator* generator, Var argument, Var resolve = nullptr, Var reject = nullptr, bool isReject = false);
         JavascriptPromiseCapabilitiesExecutorFunction* CreatePromiseCapabilitiesExecutorFunction(JavascriptMethod entryPoint, JavascriptPromiseCapability* capability);
@@ -1068,7 +1076,6 @@ namespace Js
         template<> JavascriptString* CreateStringFromCppLiteral(const char16 (&value)[1]) const; // Specialization for empty string
         template<> JavascriptString* CreateStringFromCppLiteral(const char16 (&value)[2]) const; // Specialization for single-char strings
         PropertyString* CreatePropertyString(const Js::PropertyRecord* propertyRecord);
-        PropertyString* CreatePropertyString(const Js::PropertyRecord* propertyRecord, ArenaAllocator *arena);
 
         JavascriptVariantDate* CreateVariantDate(const double value);
 
@@ -1076,7 +1083,9 @@ namespace Js
         JavascriptBooleanObject* CreateBooleanObject();
         JavascriptNumberObject* CreateNumberObjectWithCheck(double value);
         JavascriptNumberObject* CreateNumberObject(Var number);
+#ifdef ENABLE_SIMDJS
         JavascriptSIMDObject* CreateSIMDObject(Var simdValue, TypeId typeDescriptor);
+#endif
         JavascriptStringObject* CreateStringObject(JavascriptString* value);
         JavascriptStringObject* CreateStringObject(const char16* value, charcount_t length);
         JavascriptSymbolObject* CreateSymbolObject(JavascriptSymbol* value);
@@ -1098,6 +1107,10 @@ namespace Js
         JavascriptFunction* EnsurePromiseThenFunction();
         JavascriptFunction* EnsureGeneratorNextFunction();
         JavascriptFunction* EnsureGeneratorThrowFunction();
+        JavascriptFunction* EnsureArrayPrototypeForEachFunction();
+        JavascriptFunction* EnsureArrayPrototypeKeysFunction();
+        JavascriptFunction* EnsureArrayPrototypeEntriesFunction();
+        JavascriptFunction* EnsureArrayPrototypeValuesFunction();
         JavascriptFunction* EnsureJSONStringifyFunction();
         JavascriptFunction* EnsureObjectFreezeFunction();
 
@@ -1185,7 +1198,7 @@ namespace Js
         void SetArrayObjectHasUserDefinedSpecies(bool val) { arrayObjectHasUserDefinedSpecies = val; }
 
         FunctionBody* GetFakeGlobalFuncForUndefer()const { return this->fakeGlobalFuncForUndefer; }
-        void SetFakeGlobalFuncForUndefer(FunctionBody* functionBody) { this->fakeGlobalFuncForUndefer = functionBody; }        
+        void SetFakeGlobalFuncForUndefer(FunctionBody* functionBody) { this->fakeGlobalFuncForUndefer = functionBody; }
 
         ModuleRecordList* EnsureModuleRecordList();
         SourceTextModuleRecord* GetModuleRecord(uint moduleId);
@@ -1205,8 +1218,8 @@ namespace Js
         static void PrecalculateArrayAllocationBuckets();
 
 #define STANDARD_INIT(name) \
-        static void __cdecl Initialize##name##Constructor(DynamicObject* arrayConstructor, DeferredTypeHandlerBase * typeHandler, DeferredInitializeMode mode); \
-        static void __cdecl Initialize##name##Prototype(DynamicObject* arrayPrototype, DeferredTypeHandlerBase * typeHandler, DeferredInitializeMode mode);
+        static bool __cdecl Initialize##name##Constructor(DynamicObject* arrayConstructor, DeferredTypeHandlerBase * typeHandler, DeferredInitializeMode mode); \
+        static bool __cdecl Initialize##name##Prototype(DynamicObject* arrayPrototype, DeferredTypeHandlerBase * typeHandler, DeferredInitializeMode mode);
 
         STANDARD_INIT(Array);
         STANDARD_INIT(SharedArrayBuffer);
@@ -1256,57 +1269,60 @@ namespace Js
 
 #undef STANDARD_INIT
 
-        static void __cdecl InitializeAtomicsObject(DynamicObject* atomicsObject, DeferredTypeHandlerBase * typeHandler, DeferredInitializeMode mode);
+        static bool __cdecl InitializeAtomicsObject(DynamicObject* atomicsObject, DeferredTypeHandlerBase * typeHandler, DeferredInitializeMode mode);
 
-        static void __cdecl InitializeInt64ArrayPrototype(DynamicObject* prototype, DeferredTypeHandlerBase * typeHandler, DeferredInitializeMode mode);
-        static void __cdecl InitializeUint64ArrayPrototype(DynamicObject* prototype, DeferredTypeHandlerBase * typeHandler, DeferredInitializeMode mode);
-        static void __cdecl InitializeBoolArrayPrototype(DynamicObject* prototype, DeferredTypeHandlerBase * typeHandler, DeferredInitializeMode mode);
-        static void __cdecl InitializeCharArrayPrototype(DynamicObject* prototype, DeferredTypeHandlerBase * typeHandler, DeferredInitializeMode mode);
+        static bool __cdecl InitializeInt64ArrayPrototype(DynamicObject* prototype, DeferredTypeHandlerBase * typeHandler, DeferredInitializeMode mode);
+        static bool __cdecl InitializeUint64ArrayPrototype(DynamicObject* prototype, DeferredTypeHandlerBase * typeHandler, DeferredInitializeMode mode);
+        static bool __cdecl InitializeBoolArrayPrototype(DynamicObject* prototype, DeferredTypeHandlerBase * typeHandler, DeferredInitializeMode mode);
+        static bool __cdecl InitializeCharArrayPrototype(DynamicObject* prototype, DeferredTypeHandlerBase * typeHandler, DeferredInitializeMode mode);
 
         void InitializeComplexThings();
         void InitializeStaticValues();
-        static void __cdecl InitializeMathObject(DynamicObject* mathObject, DeferredTypeHandlerBase * typeHandler, DeferredInitializeMode mode);
+        static bool __cdecl InitializeMathObject(DynamicObject* mathObject, DeferredTypeHandlerBase * typeHandler, DeferredInitializeMode mode);
 #ifdef ENABLE_WASM
-        static void __cdecl InitializeWebAssemblyObject(DynamicObject* WasmObject, DeferredTypeHandlerBase * typeHandler, DeferredInitializeMode mode);
+        static bool __cdecl InitializeWebAssemblyObject(DynamicObject* WasmObject, DeferredTypeHandlerBase * typeHandler, DeferredInitializeMode mode);
 #endif
+
+#ifdef ENABLE_SIMDJS
         // SIMD_JS
-        static void __cdecl InitializeSIMDObject(DynamicObject* simdObject, DeferredTypeHandlerBase * typeHandler, DeferredInitializeMode mode);
-        static void __cdecl InitializeSIMDOpCodeMaps();
+        static bool __cdecl InitializeSIMDObject(DynamicObject* simdObject, DeferredTypeHandlerBase * typeHandler, DeferredInitializeMode mode);
+        static bool __cdecl InitializeSIMDOpCodeMaps();
 
         template<typename SIMDTypeName>
         static void SIMDPrototypeInitHelper(DynamicObject* simdPrototype, JavascriptLibrary* library, JavascriptFunction* constructorFn, JavascriptString* strLiteral);
 
-        static void __cdecl InitializeSIMDBool8x16Prototype(DynamicObject* simdPrototype, DeferredTypeHandlerBase * typeHandler, DeferredInitializeMode mode);
-        static void __cdecl InitializeSIMDBool16x8Prototype(DynamicObject* simdPrototype, DeferredTypeHandlerBase * typeHandler, DeferredInitializeMode mode);
-        static void __cdecl InitializeSIMDBool32x4Prototype(DynamicObject* simdPrototype, DeferredTypeHandlerBase * typeHandler, DeferredInitializeMode mode);
-        static void __cdecl InitializeSIMDInt8x16Prototype(DynamicObject* simdPrototype, DeferredTypeHandlerBase * typeHandler, DeferredInitializeMode mode);
-        static void __cdecl InitializeSIMDInt16x8Prototype(DynamicObject* simdPrototype, DeferredTypeHandlerBase * typeHandler, DeferredInitializeMode mode);
-        static void __cdecl InitializeSIMDInt32x4Prototype(DynamicObject* simdPrototype, DeferredTypeHandlerBase * typeHandler, DeferredInitializeMode mode);
-        static void __cdecl InitializeSIMDUint8x16Prototype(DynamicObject* simdPrototype, DeferredTypeHandlerBase * typeHandler, DeferredInitializeMode mode);
-        static void __cdecl InitializeSIMDUint16x8Prototype(DynamicObject* simdPrototype, DeferredTypeHandlerBase * typeHandler, DeferredInitializeMode mode);
-        static void __cdecl InitializeSIMDUint32x4Prototype(DynamicObject* simdPrototype, DeferredTypeHandlerBase * typeHandler, DeferredInitializeMode mode);
-        static void __cdecl InitializeSIMDFloat32x4Prototype(DynamicObject* simdPrototype, DeferredTypeHandlerBase * typeHandler, DeferredInitializeMode mode);
-        static void __cdecl InitializeSIMDFloat64x2Prototype(DynamicObject* simdPrototype, DeferredTypeHandlerBase * typeHandler, DeferredInitializeMode mode);
+        static bool __cdecl InitializeSIMDBool8x16Prototype(DynamicObject* simdPrototype, DeferredTypeHandlerBase * typeHandler, DeferredInitializeMode mode);
+        static bool __cdecl InitializeSIMDBool16x8Prototype(DynamicObject* simdPrototype, DeferredTypeHandlerBase * typeHandler, DeferredInitializeMode mode);
+        static bool __cdecl InitializeSIMDBool32x4Prototype(DynamicObject* simdPrototype, DeferredTypeHandlerBase * typeHandler, DeferredInitializeMode mode);
+        static bool __cdecl InitializeSIMDInt8x16Prototype(DynamicObject* simdPrototype, DeferredTypeHandlerBase * typeHandler, DeferredInitializeMode mode);
+        static bool __cdecl InitializeSIMDInt16x8Prototype(DynamicObject* simdPrototype, DeferredTypeHandlerBase * typeHandler, DeferredInitializeMode mode);
+        static bool __cdecl InitializeSIMDInt32x4Prototype(DynamicObject* simdPrototype, DeferredTypeHandlerBase * typeHandler, DeferredInitializeMode mode);
+        static bool __cdecl InitializeSIMDUint8x16Prototype(DynamicObject* simdPrototype, DeferredTypeHandlerBase * typeHandler, DeferredInitializeMode mode);
+        static bool __cdecl InitializeSIMDUint16x8Prototype(DynamicObject* simdPrototype, DeferredTypeHandlerBase * typeHandler, DeferredInitializeMode mode);
+        static bool __cdecl InitializeSIMDUint32x4Prototype(DynamicObject* simdPrototype, DeferredTypeHandlerBase * typeHandler, DeferredInitializeMode mode);
+        static bool __cdecl InitializeSIMDFloat32x4Prototype(DynamicObject* simdPrototype, DeferredTypeHandlerBase * typeHandler, DeferredInitializeMode mode);
+        static bool __cdecl InitializeSIMDFloat64x2Prototype(DynamicObject* simdPrototype, DeferredTypeHandlerBase * typeHandler, DeferredInitializeMode mode);
+#endif
 
-        static void __cdecl InitializeJSONObject(DynamicObject* JSONObject, DeferredTypeHandlerBase * typeHandler, DeferredInitializeMode mode);
-        static void __cdecl InitializeEngineInterfaceObject(DynamicObject* engineInterface, DeferredTypeHandlerBase * typeHandler, DeferredInitializeMode mode);
-        static void __cdecl InitializeReflectObject(DynamicObject* reflectObject, DeferredTypeHandlerBase * typeHandler, DeferredInitializeMode mode);
+        static bool __cdecl InitializeJSONObject(DynamicObject* JSONObject, DeferredTypeHandlerBase * typeHandler, DeferredInitializeMode mode);
+        static bool __cdecl InitializeEngineInterfaceObject(DynamicObject* engineInterface, DeferredTypeHandlerBase * typeHandler, DeferredInitializeMode mode);
+        static bool __cdecl InitializeReflectObject(DynamicObject* reflectObject, DeferredTypeHandlerBase * typeHandler, DeferredInitializeMode mode);
 #ifdef ENABLE_INTL_OBJECT
-        static void __cdecl InitializeIntlObject(DynamicObject* IntlEngineObject, DeferredTypeHandlerBase * typeHandler, DeferredInitializeMode mode);
+        static bool __cdecl InitializeIntlObject(DynamicObject* IntlEngineObject, DeferredTypeHandlerBase * typeHandler, DeferredInitializeMode mode);
 #endif
 #ifdef ENABLE_PROJECTION
         void InitializeWinRTPromiseConstructor();
 #endif
 
-        static void __cdecl InitializeIteratorPrototype(DynamicObject* iteratorPrototype, DeferredTypeHandlerBase * typeHandler, DeferredInitializeMode mode);
-        static void __cdecl InitializeArrayIteratorPrototype(DynamicObject* arrayIteratorPrototype, DeferredTypeHandlerBase * typeHandler, DeferredInitializeMode mode);
-        static void __cdecl InitializeMapIteratorPrototype(DynamicObject* mapIteratorPrototype, DeferredTypeHandlerBase * typeHandler, DeferredInitializeMode mode);
-        static void __cdecl InitializeSetIteratorPrototype(DynamicObject* setIteratorPrototype, DeferredTypeHandlerBase * typeHandler, DeferredInitializeMode mode);
-        static void __cdecl InitializeStringIteratorPrototype(DynamicObject* stringIteratorPrototype, DeferredTypeHandlerBase * typeHandler, DeferredInitializeMode mode);
+        static bool __cdecl InitializeIteratorPrototype(DynamicObject* iteratorPrototype, DeferredTypeHandlerBase * typeHandler, DeferredInitializeMode mode);
+        static bool __cdecl InitializeArrayIteratorPrototype(DynamicObject* arrayIteratorPrototype, DeferredTypeHandlerBase * typeHandler, DeferredInitializeMode mode);
+        static bool __cdecl InitializeMapIteratorPrototype(DynamicObject* mapIteratorPrototype, DeferredTypeHandlerBase * typeHandler, DeferredInitializeMode mode);
+        static bool __cdecl InitializeSetIteratorPrototype(DynamicObject* setIteratorPrototype, DeferredTypeHandlerBase * typeHandler, DeferredInitializeMode mode);
+        static bool __cdecl InitializeStringIteratorPrototype(DynamicObject* stringIteratorPrototype, DeferredTypeHandlerBase * typeHandler, DeferredInitializeMode mode);
 
-        static void __cdecl InitializeGeneratorPrototype(DynamicObject* generatorPrototype, DeferredTypeHandlerBase * typeHandler, DeferredInitializeMode mode);
+        static bool __cdecl InitializeGeneratorPrototype(DynamicObject* generatorPrototype, DeferredTypeHandlerBase * typeHandler, DeferredInitializeMode mode);
 
-        static void __cdecl InitializeAsyncFunction(DynamicObject *function, DeferredTypeHandlerBase * typeHandler, DeferredInitializeMode mode);
+        static bool __cdecl InitializeAsyncFunction(DynamicObject *function, DeferredTypeHandlerBase * typeHandler, DeferredInitializeMode mode);
 
         RuntimeFunction* CreateBuiltinConstructor(FunctionInfo * functionInfo, DynamicTypeHandler * typeHandler, DynamicObject* prototype = nullptr);
         RuntimeFunction* DefaultCreateFunction(FunctionInfo * functionInfo, int length, DynamicObject * prototype, DynamicType * functionType, PropertyId nameId);
@@ -1317,9 +1333,9 @@ namespace Js
         JavascriptString* CreateEmptyString();
 
 
-        static void __cdecl InitializeGeneratorFunction(DynamicObject* function, DeferredTypeHandlerBase * typeHandler, DeferredInitializeMode mode);
+        static bool __cdecl InitializeGeneratorFunction(DynamicObject* function, DeferredTypeHandlerBase * typeHandler, DeferredInitializeMode mode);
         template<bool addPrototype>
-        static void __cdecl InitializeFunction(DynamicObject* function, DeferredTypeHandlerBase * typeHandler, DeferredInitializeMode mode);
+        static bool __cdecl InitializeFunction(DynamicObject* function, DeferredTypeHandlerBase * typeHandler, DeferredInitializeMode mode);
 
         static size_t const LibraryFunctionArgC[BuiltinFunction::Count + 1];
         static int const LibraryFunctionFlags[BuiltinFunction::Count + 1];   // returns enum BuiltInFlags.
@@ -1327,9 +1343,6 @@ namespace Js
         static char16 const * const LibraryFunctionName[BuiltinFunction::Count + 1];
 #endif
 
-        JavascriptFunction* EnsureArrayPrototypeValuesFunction();
-
-
     public:
         virtual void Finalize(bool isShutdown) override;
 
@@ -1381,7 +1394,9 @@ namespace Js
         HRESULT ProfilerRegisterProxy();
         HRESULT ProfilerRegisterReflect();
         HRESULT ProfilerRegisterGenerator();
+#ifdef ENABLE_SIMDJS
         HRESULT ProfilerRegisterSIMD();
+#endif
         HRESULT ProfilerRegisterAtomics();
 
 #ifdef IR_VIEWER
