@@ -8,6 +8,7 @@
 #ifdef ENABLE_WASM
 
 #include "../WasmReader/WasmReaderPch.h"
+#include "Language/WebAssemblySource.h"
 
 namespace Js
 {
@@ -33,6 +34,7 @@ WebAssemblyModule::WebAssemblyModule(Js::ScriptContext* scriptContext, const byt
     m_signaturesCount(0),
     m_startFuncIndex(Js::Constants::UninitializedValue),
     m_binaryBuffer(binaryBuffer),
+    m_binaryBufferLength(binaryBufferLength),
     m_customSections(nullptr)
 {
     //the first elm is the number of Vars in front of I32; makes for a nicer offset computation
@@ -82,11 +84,9 @@ WebAssemblyModule::NewInstance(RecyclableObject* function, CallInfo callInfo, ..
     {
         JavascriptError::ThrowTypeError(scriptContext, WASMERR_NeedBufferSource);
     }
-    BYTE* buffer;
-    uint byteLength;
-    WebAssembly::ReadBufferSource(args[1], scriptContext, &buffer, &byteLength);
-
-    return CreateModule(scriptContext, buffer, byteLength);
+    
+    WebAssemblySource src(args[1], true, scriptContext);
+    return CreateModule(scriptContext, &src);
 }
 
 Var
@@ -171,13 +171,10 @@ Var WebAssemblyModule::EntryCustomSections(RecyclableObject* function, CallInfo
     {
         JavascriptError::ThrowTypeError(scriptContext, WASMERR_NeedModule);
     }
-    if (args.Info.Count < 3)
-    {
-        JavascriptError::ThrowTypeErrorVar(scriptContext, JSERR_NeedString, _u("sectionName"));
-    }
+    Var sectionNameVar = args.Info.Count > 2 ? args[2] : scriptContext->GetLibrary()->GetUndefined();
 
     WebAssemblyModule * module = WebAssemblyModule::FromVar(args[1]);
-    JavascriptString * sectionName = JavascriptConversion::ToString(args[2], scriptContext);
+    JavascriptString * sectionName = JavascriptConversion::ToString(sectionNameVar, scriptContext);
     const char16* sectionNameBuf = sectionName->GetString();
     charcount_t sectionNameLength = sectionName->GetLength();
 
@@ -206,9 +203,9 @@ Var WebAssemblyModule::EntryCustomSections(RecyclableObject* function, CallInfo
 WebAssemblyModule *
 WebAssemblyModule::CreateModule(
     ScriptContext* scriptContext,
-    const byte* buffer,
-    const uint lengthBytes)
+    WebAssemblySource* src)
 {
+    Assert(src);
     AutoProfilingPhase wasmPhase(scriptContext, Js::WasmBytecodePhase);
     Unused(wasmPhase);
 
@@ -217,33 +214,7 @@ WebAssemblyModule::CreateModule(
     Js::FunctionBody * currentBody = nullptr;
     try
     {
-        Js::AutoDynamicCodeReference dynamicFunctionReference(scriptContext);
-        SourceContextInfo * sourceContextInfo = scriptContext->CreateSourceContextInfo(scriptContext->GetNextSourceContextId(), nullptr, 0, nullptr);
-        SRCINFO si = {
-            /* sourceContextInfo   */ sourceContextInfo,
-            /* dlnHost             */ 0,
-            /* ulColumnHost        */ 0,
-            /* lnMinHost           */ 0,
-            /* ichMinHost          */ 0,
-            /* ichLimHost          */ 0,
-            /* ulCharOffset        */ 0,
-            /* mod                 */ 0,
-            /* grfsi               */ 0
-        };
-
-        // copy buffer so external changes to it don't cause issues when defer parsing
-        byte* newBuffer = RecyclerNewArrayLeaf(scriptContext->GetRecycler(), byte, lengthBytes);
-        js_memcpy_s(newBuffer, lengthBytes, buffer, lengthBytes);
-
-        // Note: We don't have real "source info" for Wasm. Following are just placeholders.
-        // Hack: Wasm handles debugging differently. Fake this as "LibraryCode" so that
-        // normal script debugging code ignores this source info and its functions.
-        const int32 cchLength = static_cast<int32>(lengthBytes / sizeof(char16));
-        Js::Utf8SourceInfo* utf8SourceInfo = Utf8SourceInfo::NewWithNoCopy(
-            scriptContext, (LPCUTF8)newBuffer, cchLength, lengthBytes, &si, /*isLibraryCode*/true);
-        scriptContext->SaveSourceNoCopy(utf8SourceInfo, cchLength, /*isCesu8*/false);
-
-        Wasm::WasmModuleGenerator bytecodeGen(scriptContext, utf8SourceInfo, newBuffer, lengthBytes);
+        Wasm::WasmModuleGenerator bytecodeGen(scriptContext, src);
 
         webAssemblyModule = bytecodeGen.GenerateModule();
 
@@ -261,25 +232,33 @@ WebAssemblyModule::CreateModule(
     }
     catch (Wasm::WasmCompilationException& ex)
     {
-        Wasm::WasmCompilationException newEx = ex;
+        // The reason that we use GetTempErrorMessageRef here, instead of just doing simply
+        // ReleaseErrorMessage and grabbing it away from the WasmCompilationException, is a
+        // slight niggle in the lifetime of the message buffer. The normal error throw..Var
+        // routines don't actually do anything to clean up their varargs - likely good, due
+        // to the variety of situations that we want to use them in. However, this fails to
+        // clean up strings if they're not cleaned up by some destructor. By handling these
+        // error strings not by grabbing them away from the WasmCompilationException, I can
+        // have the WasmCompilationException destructor clean up the strings when we throw,
+        // by which point SetErrorMessage will already have copied out what it needs.
+        BSTR originalMessage = ex.GetTempErrorMessageRef();
         if (currentBody != nullptr)
         {
-            char16* originalMessage = ex.ReleaseErrorMessage();
-            intptr_t offset = readerInfo->m_module->GetReader()->GetCurrentOffset();
-            intptr_t start = readerInfo->m_funcInfo->m_readerInfo.startOffset;
-            uint32 size = readerInfo->m_funcInfo->m_readerInfo.size;
+            Wasm::BinaryLocation location = readerInfo->m_module->GetReader()->GetCurrentLocation();
 
-            newEx = Wasm::WasmCompilationException(
-                _u("function %s at offset %d/%d: %s"),
+            originalMessage = ex.ReleaseErrorMessage();
+            ex = Wasm::WasmCompilationException(
+                _u("function %s at offset %u/%u (0x%x/0x%x): %s"),
                 currentBody->GetDisplayName(),
-                offset - start,
-                size,
+                location.offset, location.size,
+                location.offset, location.size,
                 originalMessage
             );
             currentBody->GetAsmJsFunctionInfo()->SetWasmReaderInfo(nullptr);
             SysFreeString(originalMessage);
+            originalMessage = ex.GetTempErrorMessageRef();
         }
-        JavascriptError::ThrowWebAssemblyCompileErrorVar(scriptContext, WASMERR_WasmCompileError, newEx.ReleaseErrorMessage());
+        JavascriptError::ThrowWebAssemblyCompileErrorVar(scriptContext, WASMERR_WasmCompileError, originalMessage);
     }
 
     return webAssemblyModule;
@@ -289,30 +268,15 @@ WebAssemblyModule::CreateModule(
 bool
 WebAssemblyModule::ValidateModule(
     ScriptContext* scriptContext,
-    const byte* buffer,
-    const uint lengthBytes)
+    WebAssemblySource* src)
 {
+    Assert(src);
     AutoProfilingPhase wasmPhase(scriptContext, Js::WasmBytecodePhase);
     Unused(wasmPhase);
 
     try
     {
-        Js::AutoDynamicCodeReference dynamicFunctionReference(scriptContext);
-        SRCINFO const * srcInfo = scriptContext->Cache()->noContextGlobalSourceInfo;
-
-        // review: unsure if we need copy here, but seems safer to do it
-        byte* newBuffer = RecyclerNewArrayLeaf(scriptContext->GetRecycler(), byte, lengthBytes);
-        js_memcpy_s(newBuffer, lengthBytes, buffer, lengthBytes);
-
-        // Note: We don't have real "source info" for Wasm. Following are just placeholders.
-        // Hack: Wasm handles debugging differently. Fake this as "LibraryCode" so that
-        // normal script debugging code ignores this source info and its functions.
-        const int32 cchLength = static_cast<int32>(lengthBytes / sizeof(char16));
-        Js::Utf8SourceInfo* utf8SourceInfo = Utf8SourceInfo::NewWithNoCopy(
-            scriptContext, (LPCUTF8)newBuffer, cchLength, lengthBytes, srcInfo, /*isLibraryCode*/true);
-        scriptContext->SaveSourceNoCopy(utf8SourceInfo, cchLength, /*isCesu8*/false);
-
-        Wasm::WasmModuleGenerator bytecodeGen(scriptContext, utf8SourceInfo, (byte*)newBuffer, lengthBytes);
+        Wasm::WasmModuleGenerator bytecodeGen(scriptContext, src);
 
         WebAssemblyModule * webAssemblyModule = bytecodeGen.GenerateModule();
 
@@ -321,9 +285,7 @@ WebAssemblyModule::ValidateModule(
             Js::FunctionBody * body = webAssemblyModule->GetWasmFunctionInfo(i)->GetBody();
             Wasm::WasmReaderInfo * readerInfo = body->GetAsmJsFunctionInfo()->GetWasmReaderInfo();
 
-            // TODO: avoid actually generating bytecode here
-            Wasm::WasmBytecodeGenerator::GenerateFunctionBytecode(scriptContext, readerInfo);
-
+            Wasm::WasmBytecodeGenerator::ValidateFunction(scriptContext, readerInfo);
 #if ENABLE_DEBUG_CONFIG_OPTIONS
             if (PHASE_ON(WasmValidatePrejitPhase, body))
             {
@@ -377,7 +339,15 @@ WebAssemblyModule::InitializeMemory(uint32 minPage, uint32 maxPage)
 
     if (maxPage < minPage)
     {
-        throw Wasm::WasmCompilationException(_u("Memory: MaxPage (%d) must be greater than MinPage (%d)"), maxPage, minPage);
+        throw Wasm::WasmCompilationException(_u("Memory: MaxPage (%u) must be greater than MinPage (%u)"), maxPage, minPage);
+    }
+    auto minPageTooBig = [minPage] {
+        throw Wasm::WasmCompilationException(_u("Memory: Unable to allocate minimum pages (%u)"), minPage);
+    };
+    uint32 minBytes = UInt32Math::Mul<WebAssembly::PageSize>(minPage, minPageTooBig);
+    if (minBytes > ArrayBuffer::MaxArrayBufferLength)
+    {
+        minPageTooBig();
     }
     m_hasMemory = true;
     m_memoryInitSize = minPage;
@@ -447,12 +417,6 @@ WebAssemblyModule::CreateTable() const
     return WebAssemblyTable::Create(m_tableInitSize, m_tableMaxSize, GetScriptContext());
 }
 
-bool
-WebAssemblyModule::IsValidTableImport(const WebAssemblyTable * table) const
-{
-    return m_tableImport && table->GetInitialLength() >= m_tableInitSize && table->GetMaximumLength() <= m_tableMaxSize;
-}
-
 uint32
 WebAssemblyModule::GetWasmFunctionCount() const
 {
@@ -480,6 +444,102 @@ WebAssemblyModule::GetWasmFunctionInfo(uint index) const
     return m_functionsInfo->Item(index);
 }
 
+void
+WebAssemblyModule::SwapWasmFunctionInfo(uint i1, uint i2)
+{
+    Wasm::WasmFunctionInfo* f1 = GetWasmFunctionInfo(i1);
+    Wasm::WasmFunctionInfo* f2 = GetWasmFunctionInfo(i2);
+    m_functionsInfo->SetItem(i1, f2);
+    m_functionsInfo->SetItem(i2, f1);
+}
+
+#if ENABLE_DEBUG_CONFIG_OPTIONS
+void
+WebAssemblyModule::AttachCustomInOutTracingReader(Wasm::WasmFunctionInfo* func, uint callIndex)
+{
+    Wasm::WasmFunctionInfo* calledFunc = GetWasmFunctionInfo(callIndex);
+    Wasm::WasmSignature* signature = calledFunc->GetSignature();
+    if (!calledFunc->GetSignature()->IsEquivalent(signature))
+    {
+        throw Wasm::WasmCompilationException(_u("InOut tracing reader signature mismatch"));
+    }
+    // Create the custom reader to generate the import thunk
+    Wasm::WasmCustomReader* customReader = Anew(&m_alloc, Wasm::WasmCustomReader, &m_alloc);
+    // Print the function name we are calling
+    {
+        Wasm::WasmNode nameNode;
+        nameNode.op = Wasm::wbI32Const;
+        nameNode.cnst.i32 = callIndex;
+        customReader->AddNode(nameNode);
+        nameNode.op = Wasm::wbPrintFuncName;
+        customReader->AddNode(nameNode);
+    }
+
+    for (Js::ArgSlot iParam = 0; iParam < signature->GetParamCount(); iParam++)
+    {
+        Wasm::WasmNode node;
+        node.op = Wasm::wbGetLocal;
+        node.var.num = iParam;
+        customReader->AddNode(node);
+
+        Wasm::Local param = signature->GetParam(iParam);
+        switch (param)
+        {
+        case Wasm::Local::I32: node.op = Wasm::wbPrintI32; break;
+        case Wasm::Local::I64: node.op = Wasm::wbPrintI64; break;
+        case Wasm::Local::F32: node.op = Wasm::wbPrintF32; break;
+        case Wasm::Local::F64: node.op = Wasm::wbPrintF64; break;
+        default:
+            throw Wasm::WasmCompilationException(_u("Unknown param type"));
+        }
+        customReader->AddNode(node);
+
+        if (iParam < signature->GetParamCount() - 1)
+        {
+            node.op = Wasm::wbPrintArgSeparator;
+            customReader->AddNode(node);
+        }
+    }
+
+    Wasm::WasmNode beginNode;
+    beginNode.op = Wasm::wbPrintBeginCall;
+    customReader->AddNode(beginNode);
+
+    Wasm::WasmNode callNode;
+    callNode.op = Wasm::wbCall;
+    callNode.call.num = callIndex;
+    callNode.call.funcType = Wasm::FunctionIndexTypes::Function;
+    customReader->AddNode(callNode);
+
+    Wasm::WasmTypes::WasmType returnType = signature->GetResultType();
+    Wasm::WasmNode endNode;
+    endNode.op = Wasm::wbI32Const;
+    endNode.cnst.i32 = returnType;
+    customReader->AddNode(endNode);
+    endNode.op = Wasm::wbPrintEndCall;
+    customReader->AddNode(endNode);
+
+    if (returnType != Wasm::WasmTypes::Void)
+    {
+        Wasm::WasmNode node;
+        switch (returnType)
+        {
+        case Wasm::WasmTypes::I32: node.op = Wasm::wbPrintI32; break;
+        case Wasm::WasmTypes::I64: node.op = Wasm::wbPrintI64; break;
+        case Wasm::WasmTypes::F32: node.op = Wasm::wbPrintF32; break;
+        case Wasm::WasmTypes::F64: node.op = Wasm::wbPrintF64; break;
+        default:
+            throw Wasm::WasmCompilationException(_u("Unknown return type"));
+        }
+        customReader->AddNode(node);
+    }
+    endNode.op = Wasm::wbPrintNewLine;
+    customReader->AddNode(endNode);
+
+    func->SetCustomReader(customReader);
+}
+#endif
+
 void
 WebAssemblyModule::AllocateFunctionExports(uint32 entries)
 {
@@ -533,7 +593,7 @@ WebAssemblyModule::AddFunctionImport(uint32 sigId, const char16* modName, uint32
     Wasm::WasmFunctionInfo* funcInfo = AddWasmFunctionInfo(signature);
     // Create the custom reader to generate the import thunk
     Wasm::WasmCustomReader* customReader = Anew(&m_alloc, Wasm::WasmCustomReader, &m_alloc);
-    for (uint32 iParam = 0; iParam < signature->GetParamCount(); iParam++)
+    for (uint32 iParam = 0; iParam < (uint32)signature->GetParamCount(); iParam++)
     {
         Wasm::WasmNode node;
         node.op = Wasm::wbGetLocal;
@@ -623,11 +683,11 @@ WebAssemblyModule::GetOffsetFromInit(const Wasm::WasmNode& initExpr, const WebAs
     {
         ValidateInitExportForOffset(initExpr);
     }
-    catch (Wasm::WasmCompilationException &e)
+    catch (Wasm::WasmCompilationException&)
     {
         // Should have been checked at compile time
         Assert(UNREACHED);
-        throw e;
+        throw;
     }
 
     uint offset = 0;
@@ -878,4 +938,4 @@ WebAssemblyModule::GetCustomSection(uint32 index) const
 
 } // namespace Js
 
-#endif
\ No newline at end of file
+#endif
