@@ -11,20 +11,20 @@ namespace Wasm
     class WasmElementSegment
     {
     public:
-        WasmElementSegment(ArenaAllocator * alloc, const UINT32 index, const WasmNode initExpr, const UINT32 numElem);
-        void AddElement(const UINT32 funcIndex, const Js::WebAssemblyModule& module);
-        UINT32 GetElement(const UINT32 tableIndex) const;
-        UINT32 GetNumElements() const { return m_numElem; }
+        WasmElementSegment(ArenaAllocator* alloc, const uint32 index, const WasmNode initExpr, const uint32 numElem);
+        void AddElement(const uint32 funcIndex);
+        uint32 GetElement(const uint32 tableIndex) const;
+        uint32 GetNumElements() const { return m_numElem; }
         WasmNode GetOffsetExpr() const { return m_offsetExpr; }
     private:
         ArenaAllocator* m_alloc;
-        UINT32 m_index;
+        uint32 m_index;
         const WasmNode m_offsetExpr;
-        UINT32 m_numElem;
-        UINT32 m_offset;
-        UINT32 m_elemIdx;
-        UINT32* m_elems;
+        uint32 m_numElem;
+        uint32 m_offset;
+        uint32 m_elemIdx;
+        uint32* m_elems;
 
-        void Init(const Js::WebAssemblyModule& module);
+        void Init();
     };
 } // Namespace Wasm
