@@ -556,9 +556,6 @@ DECLARE_GETOBJECT(ProxyConstructor,
                   globalConstructor[GlobalType::Proxy])
 DECLARE_GETOBJECT(MapConstructor,
                   globalConstructor[GlobalType::Map])
-DECLARE_GETOBJECT(HasOwnPropertyFunction,
-                  globalPrototypeFunction[GlobalPrototypeFunction
-                    ::Object_hasOwnProperty])
 DECLARE_GETOBJECT(ToStringFunction,
                   globalPrototypeFunction[GlobalPrototypeFunction
                     ::Object_toString])
