@@ -1,51 +1,70 @@
+// Copyright Joyent, Inc. and other Node contributors.
+//
+// Permission is hereby granted, free of charge, to any person obtaining a
+// copy of this software and associated documentation files (the
+// "Software"), to deal in the Software without restriction, including
+// without limitation the rights to use, copy, modify, merge, publish,
+// distribute, sublicense, and/or sell copies of the Software, and to permit
+// persons to whom the Software is furnished to do so, subject to the
+// following conditions:
+//
+// The above copyright notice and this permission notice shall be included
+// in all copies or substantial portions of the Software.
+//
+// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
+// OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
+// MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN
+// NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM,
+// DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR
+// OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE
+// USE OR OTHER DEALINGS IN THE SOFTWARE.
+
 'use strict';
 
 const assert = require('assert').ok;
 const Stream = require('stream');
-const timers = require('timers');
 const util = require('util');
 const internalUtil = require('internal/util');
+const internalHttp = require('internal/http');
 const Buffer = require('buffer').Buffer;
 const common = require('_http_common');
 const checkIsHttpToken = common._checkIsHttpToken;
 const checkInvalidHeaderChar = common._checkInvalidHeaderChar;
+const outHeadersKey = require('internal/http').outHeadersKey;
+const async_id_symbol = process.binding('async_wrap').async_id_symbol;
+const nextTick = require('internal/process/next_tick').nextTick;
 
 const CRLF = common.CRLF;
-const trfrEncChunkExpression = common.chunkExpression;
 const debug = common.debug;
+const utcDate = internalHttp.utcDate;
 
-const upgradeExpression = /^Upgrade$/i;
-const transferEncodingExpression = /^Transfer-Encoding$/i;
-const contentLengthExpression = /^Content-Length$/i;
-const dateExpression = /^Date$/i;
-const expectExpression = /^Expect$/i;
-const trailerExpression = /^Trailer$/i;
-const connectionExpression = /^Connection$/i;
-const connCloseExpression = /(^|\W)close(\W|$)/i;
-const connUpgradeExpression = /(^|\W)upgrade(\W|$)/i;
+var RE_FIELDS =
+  /^(?:Connection|Transfer-Encoding|Content-Length|Date|Expect|Trailer|Upgrade)$/i;
+var RE_CONN_VALUES = /(?:^|\W)close|upgrade(?:$|\W)/ig;
+var RE_TE_CHUNKED = common.chunkExpression;
 
-const automaticHeaders = {
-  connection: true,
-  'content-length': true,
-  'transfer-encoding': true,
-  date: true
-};
-
-
-var dateCache;
-function utcDate() {
-  if (!dateCache) {
-    var d = new Date();
-    dateCache = d.toUTCString();
-    timers.enroll(utcDate, 1000 - d.getMilliseconds());
-    timers._unrefActive(utcDate);
-  }
-  return dateCache;
+// isCookieField performs a case-insensitive comparison of a provided string
+// against the word "cookie." This method (at least as of V8 5.4) is faster than
+// the equivalent case-insensitive regexp, even if isCookieField does not get
+// inlined.
+function isCookieField(s) {
+  if (s.length !== 6) return false;
+  var ch = s.charCodeAt(0);
+  if (ch !== 99 && ch !== 67) return false;
+  ch = s.charCodeAt(1);
+  if (ch !== 111 && ch !== 79) return false;
+  ch = s.charCodeAt(2);
+  if (ch !== 111 && ch !== 79) return false;
+  ch = s.charCodeAt(3);
+  if (ch !== 107 && ch !== 75) return false;
+  ch = s.charCodeAt(4);
+  if (ch !== 105 && ch !== 73) return false;
+  ch = s.charCodeAt(5);
+  if (ch !== 101 && ch !== 69) return false;
+  return true;
 }
-utcDate._onTimeout = function _onTimeout() {
-  dateCache = undefined;
-};
 
+function noopPendingOutput(amount) {}
 
 function OutgoingMessage() {
   Stream.call(this);
@@ -70,7 +89,9 @@ function OutgoingMessage() {
   this.shouldKeepAlive = true;
   this.useChunkedEncodingByDefault = true;
   this.sendDate = false;
-  this._removedHeader = {};
+  this._removedConnection = false;
+  this._removedContLen = false;
+  this._removedTE = false;
 
   this._contentLength = null;
   this._hasBody = true;
@@ -82,14 +103,82 @@ function OutgoingMessage() {
   this.socket = null;
   this.connection = null;
   this._header = null;
-  this._headers = null;
-  this._headerNames = {};
+  this[outHeadersKey] = null;
 
-  this._onPendingData = null;
+  this._onPendingData = noopPendingOutput;
 }
 util.inherits(OutgoingMessage, Stream);
 
 
+Object.defineProperty(OutgoingMessage.prototype, '_headers', {
+  get: function() {
+    return this.getHeaders();
+  },
+  set: function(val) {
+    if (val == null) {
+      this[outHeadersKey] = null;
+    } else if (typeof val === 'object') {
+      const headers = this[outHeadersKey] = {};
+      const keys = Object.keys(val);
+      for (var i = 0; i < keys.length; ++i) {
+        const name = keys[i];
+        headers[name.toLowerCase()] = [name, val[name]];
+      }
+    }
+  }
+});
+
+Object.defineProperty(OutgoingMessage.prototype, '_headerNames', {
+  get: function() {
+    const headers = this[outHeadersKey];
+    if (headers) {
+      const out = Object.create(null);
+      const keys = Object.keys(headers);
+      for (var i = 0; i < keys.length; ++i) {
+        const key = keys[i];
+        const val = headers[key][0];
+        out[key] = val;
+      }
+      return out;
+    } else {
+      return headers;
+    }
+  },
+  set: function(val) {
+    if (typeof val === 'object' && val !== null) {
+      const headers = this[outHeadersKey];
+      if (!headers)
+        return;
+      const keys = Object.keys(val);
+      for (var i = 0; i < keys.length; ++i) {
+        const header = headers[keys[i]];
+        if (header)
+          header[0] = val[keys[i]];
+      }
+    }
+  }
+});
+
+
+OutgoingMessage.prototype._renderHeaders = function _renderHeaders() {
+  if (this._header) {
+    throw new Error('Can\'t render headers after they are sent to the client');
+  }
+
+  var headersMap = this[outHeadersKey];
+  if (!headersMap) return {};
+
+  var headers = {};
+  var keys = Object.keys(headersMap);
+
+  for (var i = 0, l = keys.length; i < l; i++) {
+    var key = keys[i];
+    headers[headersMap[key][0]] = headersMap[key][1];
+  }
+  return headers;
+};
+
+
 exports.OutgoingMessage = OutgoingMessage;
 
 
@@ -114,12 +203,13 @@ OutgoingMessage.prototype.setTimeout = function setTimeout(msecs, callback) {
 // any messages, before ever calling this.  In that case, just skip
 // it, since something else is destroying this connection anyway.
 OutgoingMessage.prototype.destroy = function destroy(error) {
-  if (this.socket)
+  if (this.socket) {
     this.socket.destroy(error);
-  else
+  } else {
     this.once('socket', function(socket) {
       socket.destroy(error);
     });
+  }
 };
 
 
@@ -130,16 +220,21 @@ OutgoingMessage.prototype._send = function _send(data, encoding, callback) {
   // this at a lower level and in a more general way.
   if (!this._headerSent) {
     if (typeof data === 'string' &&
-        encoding !== 'hex' &&
-        encoding !== 'base64') {
+        (encoding === 'utf8' || encoding === 'latin1' || !encoding)) {
       data = this._header + data;
     } else {
-      this.output.unshift(this._header);
-      this.outputEncodings.unshift('latin1');
-      this.outputCallbacks.unshift(null);
-      this.outputSize += this._header.length;
-      if (typeof this._onPendingData === 'function')
-        this._onPendingData(this._header.length);
+      var header = this._header;
+      if (this.output.length === 0) {
+        this.output = [header];
+        this.outputEncodings = ['latin1'];
+        this.outputCallbacks = [null];
+      } else {
+        this.output.unshift(header);
+        this.outputEncodings.unshift('latin1');
+        this.outputCallbacks.unshift(null);
+      }
+      this.outputSize += header.length;
+      this._onPendingData(header.length);
     }
     this._headerSent = true;
   }
@@ -149,48 +244,39 @@ OutgoingMessage.prototype._send = function _send(data, encoding, callback) {
 
 OutgoingMessage.prototype._writeRaw = _writeRaw;
 function _writeRaw(data, encoding, callback) {
+  const conn = this.connection;
+  if (conn && conn.destroyed) {
+    // The socket was destroyed. If we're still trying to write to it,
+    // then we haven't gotten the 'close' event yet.
+    return false;
+  }
+
   if (typeof encoding === 'function') {
     callback = encoding;
     encoding = null;
   }
 
-  var connection = this.connection;
-  if (connection &&
-      connection._httpMessage === this &&
-      connection.writable &&
-      !connection.destroyed) {
+  if (conn && conn._httpMessage === this && conn.writable && !conn.destroyed) {
     // There might be pending data in the this.output buffer.
-    var outputLength = this.output.length;
-    if (outputLength > 0) {
-      this._flushOutput(connection);
-    } else if (data.length === 0) {
-      if (typeof callback === 'function')
-        process.nextTick(callback);
+    if (this.output.length) {
+      this._flushOutput(conn);
+    } else if (!data.length) {
+      if (typeof callback === 'function') {
+        nextTick(this.socket[async_id_symbol], callback);
+      }
       return true;
     }
-
     // Directly write to socket.
-    return connection.write(data, encoding, callback);
-  } else if (connection && connection.destroyed) {
-    // The socket was destroyed.  If we're still trying to write to it,
-    // then we haven't gotten the 'close' event yet.
-    return false;
-  } else {
-    // buffer, as long as we're not destroyed.
-    return this._buffer(data, encoding, callback);
+    return conn.write(data, encoding, callback);
   }
-}
-
-
-OutgoingMessage.prototype._buffer = function _buffer(data, encoding, callback) {
+  // Buffer, as long as we're not destroyed.
   this.output.push(data);
   this.outputEncodings.push(encoding);
   this.outputCallbacks.push(callback);
   this.outputSize += data.length;
-  if (typeof this._onPendingData === 'function')
-    this._onPendingData(data.length);
+  this._onPendingData(data.length);
   return false;
-};
+}
 
 
 OutgoingMessage.prototype._storeHeader = _storeHeader;
@@ -198,57 +284,76 @@ function _storeHeader(firstLine, headers) {
   // firstLine in the case of request is: 'GET /index.html HTTP/1.1\r\n'
   // in the case of response it is: 'HTTP/1.1 200 OK\r\n'
   var state = {
-    sentConnectionHeader: false,
-    sentConnectionUpgrade: false,
-    sentContentLengthHeader: false,
-    sentTransferEncodingHeader: false,
-    sentDateHeader: false,
-    sentExpect: false,
-    sentTrailer: false,
-    sentUpgrade: false,
-    messageHeader: firstLine
+    connection: false,
+    connUpgrade: false,
+    contLen: false,
+    te: false,
+    date: false,
+    expect: false,
+    trailer: false,
+    upgrade: false,
+    header: firstLine
   };
 
+  var field;
+  var key;
+  var value;
   var i;
   var j;
-  var field;
-  var value;
-  if (headers instanceof Array) {
-    for (i = 0; i < headers.length; ++i) {
+  if (headers === this[outHeadersKey]) {
+    for (key in headers) {
+      var entry = headers[key];
+      field = entry[0];
+      value = entry[1];
+
+      if (value instanceof Array) {
+        if (value.length < 2 || !isCookieField(field)) {
+          for (j = 0; j < value.length; j++)
+            storeHeader(this, state, field, value[j], false);
+          continue;
+        }
+        value = value.join('; ');
+      }
+      storeHeader(this, state, field, value, false);
+    }
+  } else if (headers instanceof Array) {
+    for (i = 0; i < headers.length; i++) {
       field = headers[i][0];
       value = headers[i][1];
 
       if (value instanceof Array) {
         for (j = 0; j < value.length; j++) {
-          storeHeader(this, state, field, value[j]);
+          storeHeader(this, state, field, value[j], true);
         }
       } else {
-        storeHeader(this, state, field, value);
+        storeHeader(this, state, field, value, true);
       }
     }
   } else if (headers) {
     var keys = Object.keys(headers);
-    for (i = 0; i < keys.length; ++i) {
+    for (i = 0; i < keys.length; i++) {
       field = keys[i];
       value = headers[field];
 
       if (value instanceof Array) {
-        for (j = 0; j < value.length; j++) {
-          storeHeader(this, state, field, value[j]);
+        if (value.length < 2 || !isCookieField(field)) {
+          for (j = 0; j < value.length; j++)
+            storeHeader(this, state, field, value[j], true);
+          continue;
         }
-      } else {
-        storeHeader(this, state, field, value);
+        value = value.join('; ');
       }
+      storeHeader(this, state, field, value, true);
     }
   }
 
   // Are we upgrading the connection?
-  if (state.sentConnectionUpgrade && state.sentUpgrade)
+  if (state.connUpgrade && state.upgrade)
     this.upgrading = true;
 
   // Date header
-  if (this.sendDate && !state.sentDateHeader) {
-    state.messageHeader += 'Date: ' + utcDate() + CRLF;
+  if (this.sendDate && !state.date) {
+    state.header += 'Date: ' + utcDate() + CRLF;
   }
 
   // Force the connection to close when the response is a 204 No Content or
@@ -271,36 +376,33 @@ function _storeHeader(firstLine, headers) {
   }
 
   // keep-alive logic
-  if (this._removedHeader.connection) {
+  if (this._removedConnection) {
     this._last = true;
     this.shouldKeepAlive = false;
-  } else if (!state.sentConnectionHeader) {
+  } else if (!state.connection) {
     var shouldSendKeepAlive = this.shouldKeepAlive &&
-        (state.sentContentLengthHeader ||
-         this.useChunkedEncodingByDefault ||
-         this.agent);
+        (state.contLen || this.useChunkedEncodingByDefault || this.agent);
     if (shouldSendKeepAlive) {
-      state.messageHeader += 'Connection: keep-alive\r\n';
+      state.header += 'Connection: keep-alive\r\n';
     } else {
       this._last = true;
-      state.messageHeader += 'Connection: close\r\n';
+      state.header += 'Connection: close\r\n';
     }
   }
 
-  if (!state.sentContentLengthHeader && !state.sentTransferEncodingHeader) {
+  if (!state.contLen && !state.te) {
     if (!this._hasBody) {
       // Make sure we don't end the 0\r\n\r\n at the end of the message.
       this.chunkedEncoding = false;
     } else if (!this.useChunkedEncodingByDefault) {
       this._last = true;
     } else {
-      if (!state.sentTrailer &&
-          !this._removedHeader['content-length'] &&
+      if (!state.trailer &&
+          !this._removedContLen &&
           typeof this._contentLength === 'number') {
-        state.messageHeader += 'Content-Length: ' + this._contentLength +
-                               '\r\n';
-      } else if (!this._removedHeader['transfer-encoding']) {
-        state.messageHeader += 'Transfer-Encoding: chunked\r\n';
+        state.header += 'Content-Length: ' + this._contentLength + CRLF;
+      } else if (!this._removedTE) {
+        state.header += 'Transfer-Encoding: chunked\r\n';
         this.chunkedEncoding = true;
       } else {
         // We should only be able to get here if both Content-Length and
@@ -311,90 +413,159 @@ function _storeHeader(firstLine, headers) {
     }
   }
 
-  this._header = state.messageHeader + CRLF;
+  this._header = state.header + CRLF;
   this._headerSent = false;
 
   // wait until the first body chunk, or close(), is sent to flush,
   // UNLESS we're sending Expect: 100-continue.
-  if (state.sentExpect) this._send('');
+  if (state.expect) this._send('');
 }
 
-function storeHeader(self, state, key, value) {
-  if (typeof key !== 'string' || !key || !checkIsHttpToken(key)) {
-    throw new TypeError(
-      'Header name must be a valid HTTP Token ["' + key + '"]');
-  }
-  if (checkInvalidHeaderChar(value)) {
-    debug('Header "%s" contains invalid characters', key);
-    throw new TypeError('The header content contains invalid characters');
-  }
-  state.messageHeader += key + ': ' + escapeHeaderValue(value) + CRLF;
-
-  if (connectionExpression.test(key)) {
-    state.sentConnectionHeader = true;
-    if (connCloseExpression.test(value)) {
-      self._last = true;
-    } else {
-      self.shouldKeepAlive = true;
+function storeHeader(self, state, key, value, validate) {
+  if (validate) {
+    if (typeof key !== 'string' || !key || !checkIsHttpToken(key)) {
+      throw new TypeError(
+        'Header name must be a valid HTTP Token ["' + key + '"]');
     }
-    if (connUpgradeExpression.test(value))
-      state.sentConnectionUpgrade = true;
-  } else if (transferEncodingExpression.test(key)) {
-    state.sentTransferEncodingHeader = true;
-    if (trfrEncChunkExpression.test(value)) self.chunkedEncoding = true;
+    if (value === undefined) {
+      throw new Error('Header "%s" value must not be undefined', key);
+    } else if (checkInvalidHeaderChar(value)) {
+      debug('Header "%s" contains invalid characters', key);
+      throw new TypeError('The header content contains invalid characters');
+    }
+  }
+  state.header += key + ': ' + escapeHeaderValue(value) + CRLF;
+  matchHeader(self, state, key, value);
+}
 
-  } else if (contentLengthExpression.test(key)) {
-    state.sentContentLengthHeader = true;
-  } else if (dateExpression.test(key)) {
-    state.sentDateHeader = true;
-  } else if (expectExpression.test(key)) {
-    state.sentExpect = true;
-  } else if (trailerExpression.test(key)) {
-    state.sentTrailer = true;
-  } else if (upgradeExpression.test(key)) {
-    state.sentUpgrade = true;
+function matchConnValue(self, state, value) {
+  var sawClose = false;
+  var m = RE_CONN_VALUES.exec(value);
+  while (m) {
+    if (m[0].length === 5)
+      sawClose = true;
+    else
+      state.connUpgrade = true;
+    m = RE_CONN_VALUES.exec(value);
+  }
+  if (sawClose)
+    self._last = true;
+  else
+    self.shouldKeepAlive = true;
+}
+
+function matchHeader(self, state, field, value) {
+  var m = RE_FIELDS.exec(field);
+  if (!m)
+    return;
+  var len = m[0].length;
+  if (len === 10) {
+    state.connection = true;
+    matchConnValue(self, state, value);
+  } else if (len === 17) {
+    state.te = true;
+    if (RE_TE_CHUNKED.test(value)) self.chunkedEncoding = true;
+  } else if (len === 14) {
+    state.contLen = true;
+  } else if (len === 4) {
+    state.date = true;
+  } else if (len === 6) {
+    state.expect = true;
+  } else if (len === 7) {
+    var ch = m[0].charCodeAt(0);
+    if (ch === 85 || ch === 117)
+      state.upgrade = true;
+    else
+      state.trailer = true;
   }
 }
 
-
-OutgoingMessage.prototype.setHeader = function setHeader(name, value) {
+function validateHeader(msg, name, value) {
   if (typeof name !== 'string' || !name || !checkIsHttpToken(name))
-    throw new TypeError(
-      'Header name must be a valid HTTP Token ["' + name + '"]');
+    throw new TypeError(`Header name must be a valid HTTP Token ["${name}"]`);
   if (value === undefined)
     throw new Error('"value" required in setHeader("' + name + '", value)');
-  if (this._header)
+  if (msg._header)
     throw new Error('Can\'t set headers after they are sent.');
   if (checkInvalidHeaderChar(value)) {
     debug('Header "%s" contains invalid characters', name);
     throw new TypeError('The header content contains invalid characters');
   }
-  if (this._headers === null)
-    this._headers = {};
+}
+OutgoingMessage.prototype.setHeader = function setHeader(name, value) {
+  validateHeader(this, name, value);
 
-  var key = name.toLowerCase();
-  this._headers[key] = value;
-  this._headerNames[key] = name;
+  if (!this[outHeadersKey])
+    this[outHeadersKey] = {};
 
-  if (automaticHeaders[key])
-    this._removedHeader[key] = false;
+  const key = name.toLowerCase();
+  this[outHeadersKey][key] = [name, value];
+
+  switch (key.length) {
+    case 10:
+      if (key === 'connection')
+        this._removedConnection = false;
+      break;
+    case 14:
+      if (key === 'content-length')
+        this._removedContLen = false;
+      break;
+    case 17:
+      if (key === 'transfer-encoding')
+        this._removedTE = false;
+      break;
+  }
 };
 
 
 OutgoingMessage.prototype.getHeader = function getHeader(name) {
-  if (arguments.length < 1) {
-    throw new Error('"name" argument is required for getHeader(name)');
+  if (typeof name !== 'string') {
+    throw new TypeError('"name" argument must be a string');
   }
 
-  if (!this._headers) return;
+  if (!this[outHeadersKey]) return;
 
-  return this._headers[name.toLowerCase()];
+  var entry = this[outHeadersKey][name.toLowerCase()];
+  if (!entry)
+    return;
+  return entry[1];
+};
+
+
+// Returns an array of the names of the current outgoing headers.
+OutgoingMessage.prototype.getHeaderNames = function getHeaderNames() {
+  return (this[outHeadersKey] ? Object.keys(this[outHeadersKey]) : []);
+};
+
+
+// Returns a shallow copy of the current outgoing headers.
+OutgoingMessage.prototype.getHeaders = function getHeaders() {
+  const headers = this[outHeadersKey];
+  const ret = Object.create(null);
+  if (headers) {
+    const keys = Object.keys(headers);
+    for (var i = 0; i < keys.length; ++i) {
+      const key = keys[i];
+      const val = headers[key][1];
+      ret[key] = val;
+    }
+  }
+  return ret;
+};
+
+
+OutgoingMessage.prototype.hasHeader = function hasHeader(name) {
+  if (typeof name !== 'string') {
+    throw new TypeError('"name" argument must be a string');
+  }
+
+  return !!(this[outHeadersKey] && this[outHeadersKey][name.toLowerCase()]);
 };
 
 
 OutgoingMessage.prototype.removeHeader = function removeHeader(name) {
-  if (arguments.length < 1) {
-    throw new Error('"name" argument is required for removeHeader(name)');
+  if (typeof name !== 'string') {
+    throw new TypeError('"name" argument must be a string');
   }
 
   if (this._header) {
@@ -403,37 +574,31 @@ OutgoingMessage.prototype.removeHeader = function removeHeader(name) {
 
   var key = name.toLowerCase();
 
-  if (key === 'date')
-    this.sendDate = false;
-  else if (automaticHeaders[key])
-    this._removedHeader[key] = true;
+  switch (key.length) {
+    case 10:
+      if (key === 'connection')
+        this._removedConnection = true;
+      break;
+    case 14:
+      if (key === 'content-length')
+        this._removedContLen = true;
+      break;
+    case 17:
+      if (key === 'transfer-encoding')
+        this._removedTE = true;
+      break;
+    case 4:
+      if (key === 'date')
+        this.sendDate = false;
+      break;
+  }
 
-  if (this._headers) {
-    delete this._headers[key];
-    delete this._headerNames[key];
+  if (this[outHeadersKey]) {
+    delete this[outHeadersKey][key];
   }
 };
 
 
-OutgoingMessage.prototype._renderHeaders = function _renderHeaders() {
-  if (this._header) {
-    throw new Error('Can\'t render headers after they are sent to the client');
-  }
-
-  var headersMap = this._headers;
-  if (!headersMap) return {};
-
-  var headers = {};
-  var keys = Object.keys(headersMap);
-  var headerNames = this._headerNames;
-
-  for (var i = 0, l = keys.length; i < l; i++) {
-    var key = keys[i];
-    headers[headerNames[key]] = headersMap[key];
-  }
-  return headers;
-};
-
 OutgoingMessage.prototype._implicitHeader = function _implicitHeader() {
   throw new Error('_implicitHeader() method is not implemented');
 };
@@ -445,25 +610,33 @@ Object.defineProperty(OutgoingMessage.prototype, 'headersSent', {
 });
 
 
+const crlf_buf = Buffer.from('\r\n');
 OutgoingMessage.prototype.write = function write(chunk, encoding, callback) {
-  if (this.finished) {
+  return write_(this, chunk, encoding, callback, false);
+};
+
+function write_(msg, chunk, encoding, callback, fromEnd) {
+  if (msg.finished) {
     var err = new Error('write after end');
-    process.nextTick(writeAfterEndNT, this, err, callback);
+    nextTick(msg.socket && msg.socket[async_id_symbol],
+             writeAfterEndNT.bind(msg),
+             err,
+             callback);
 
     return true;
   }
 
-  if (!this._header) {
-    this._implicitHeader();
+  if (!msg._header) {
+    msg._implicitHeader();
   }
 
-  if (!this._hasBody) {
+  if (!msg._hasBody) {
     debug('This type of response MUST NOT have a body. ' +
           'Ignoring write() calls.');
     return true;
   }
 
-  if (typeof chunk !== 'string' && !(chunk instanceof Buffer)) {
+  if (!fromEnd && typeof chunk !== 'string' && !(chunk instanceof Buffer)) {
     throw new TypeError('First argument must be a string or Buffer');
   }
 
@@ -472,42 +645,33 @@ OutgoingMessage.prototype.write = function write(chunk, encoding, callback) {
   // signal the user to keep writing.
   if (chunk.length === 0) return true;
 
-  var len, ret;
-  if (this.chunkedEncoding) {
-    if (typeof chunk === 'string' &&
-        encoding !== 'hex' &&
-        encoding !== 'base64' &&
-        encoding !== 'latin1') {
-      len = Buffer.byteLength(chunk, encoding);
-      chunk = len.toString(16) + CRLF + chunk + CRLF;
-      ret = this._send(chunk, encoding, callback);
-    } else {
-      // buffer, or a non-toString-friendly encoding
-      if (typeof chunk === 'string')
-        len = Buffer.byteLength(chunk, encoding);
-      else
-        len = chunk.length;
+  if (!fromEnd && msg.connection && !msg.connection.corked) {
+    msg.connection.cork();
+    process.nextTick(connectionCorkNT, msg.connection);
+  }
 
-      if (this.connection && !this.connection.corked) {
-        this.connection.cork();
-        process.nextTick(connectionCorkNT, this.connection);
-      }
-      this._send(len.toString(16), 'latin1', null);
-      this._send(crlf_buf, null, null);
-      this._send(chunk, encoding, null);
-      ret = this._send(crlf_buf, null, callback);
-    }
+  var len, ret;
+  if (msg.chunkedEncoding) {
+    if (typeof chunk === 'string')
+      len = Buffer.byteLength(chunk, encoding);
+    else
+      len = chunk.length;
+
+    msg._send(len.toString(16), 'latin1', null);
+    msg._send(crlf_buf, null, null);
+    msg._send(chunk, encoding, null);
+    ret = msg._send(crlf_buf, null, callback);
   } else {
-    ret = this._send(chunk, encoding, callback);
+    ret = msg._send(chunk, encoding, callback);
   }
 
   debug('write ret = ' + ret);
   return ret;
-};
+}
 
 
-function writeAfterEndNT(self, err, callback) {
-  self.emit('error', err);
+function writeAfterEndNT(err, callback) {
+  this.emit('error', err);
   if (callback) callback(err);
 }
 
@@ -550,56 +714,44 @@ OutgoingMessage.prototype.addTrailers = function addTrailers(headers) {
   }
 };
 
-
-const crlf_buf = Buffer.from('\r\n');
-
 function onFinish(outmsg) {
   outmsg.emit('finish');
 }
 
-OutgoingMessage.prototype.end = function end(data, encoding, callback) {
-  if (typeof data === 'function') {
-    callback = data;
-    data = null;
+OutgoingMessage.prototype.end = function end(chunk, encoding, callback) {
+  if (typeof chunk === 'function') {
+    callback = chunk;
+    chunk = null;
   } else if (typeof encoding === 'function') {
     callback = encoding;
     encoding = null;
   }
 
-  if (data && typeof data !== 'string' && !(data instanceof Buffer)) {
-    throw new TypeError('First argument must be a string or Buffer');
-  }
-
   if (this.finished) {
     return false;
   }
 
-  if (!this._header) {
-    if (data) {
-      if (typeof data === 'string')
-        this._contentLength = Buffer.byteLength(data, encoding);
-      else
-        this._contentLength = data.length;
-    } else {
-      this._contentLength = 0;
+  var uncork;
+  if (chunk) {
+    if (typeof chunk !== 'string' && !(chunk instanceof Buffer)) {
+      throw new TypeError('First argument must be a string or Buffer');
     }
+    if (!this._header) {
+      if (typeof chunk === 'string')
+        this._contentLength = Buffer.byteLength(chunk, encoding);
+      else
+        this._contentLength = chunk.length;
+    }
+    if (this.connection) {
+      this.connection.cork();
+      uncork = true;
+    }
+    write_(this, chunk, encoding, null, true);
+  } else if (!this._header) {
+    this._contentLength = 0;
     this._implicitHeader();
   }
 
-  if (data && !this._hasBody) {
-    debug('This type of response MUST NOT have a body. ' +
-          'Ignoring data passed to end().');
-    data = null;
-  }
-
-  if (this.connection && data)
-    this.connection.cork();
-
-  if (data) {
-    // Normal body write.
-    this.write(data, encoding);
-  }
-
   if (typeof callback === 'function')
     this.once('finish', callback);
 
@@ -613,10 +765,11 @@ OutgoingMessage.prototype.end = function end(data, encoding, callback) {
     ret = this._send('', 'latin1', finish);
   }
 
-  if (this.connection && data)
+  if (uncork)
     this.connection.uncork();
 
   this.finished = true;
+  this.writable = false;
 
   // There is the first message on the outgoing queue, and we've sent
   // everything to the socket.
@@ -692,8 +845,7 @@ OutgoingMessage.prototype._flushOutput = function _flushOutput(socket) {
   this.output = [];
   this.outputEncodings = [];
   this.outputCallbacks = [];
-  if (typeof this._onPendingData === 'function')
-    this._onPendingData(-this.outputSize);
+  this._onPendingData(-this.outputSize);
   this.outputSize = 0;
 
   return ret;
@@ -711,4 +863,9 @@ OutgoingMessage.prototype.flushHeaders = function flushHeaders() {
 
 OutgoingMessage.prototype.flush = internalUtil.deprecate(function() {
   this.flushHeaders();
-}, 'OutgoingMessage.flush is deprecated. Use flushHeaders instead.');
+}, 'OutgoingMessage.flush is deprecated. Use flushHeaders instead.', 'DEP0001');
+
+
+module.exports = {
+  OutgoingMessage
+};
