@@ -284,6 +284,7 @@
         return console;
       }
     });
+    setupInspectorCommandLineAPI();
   }
 
   function installInspectorConsole(globalConsole) {
@@ -312,6 +313,22 @@
     return wrappedConsole;
   }
 
+  function setupInspectorCommandLineAPI() {
+    const { addCommandLineAPI } = process.binding('inspector');
+    if (!addCommandLineAPI) return;
+
+    const Module = NativeModule.require('module');
+    const { makeRequireFunction } = NativeModule.require('internal/module');
+    const path = NativeModule.require('path');
+    const cwd = tryGetCwd(path);
+
+    const consoleAPIModule = new Module('<inspector console>');
+    consoleAPIModule.paths =
+        Module._nodeModulePaths(cwd).concat(Module.globalPaths);
+
+    addCommandLineAPI('require', makeRequireFunction(consoleAPIModule));
+  }
+
   function setupProcessFatal() {
     const async_wrap = process.binding('async_wrap');
     // Arrays containing hook flags and ids for async_hook calls.
@@ -486,6 +503,11 @@
   NativeModule._source = process.binding('natives');
   NativeModule._cache = {};
 
+  const config = process.binding('config');
+
+  if (!config.exposeHTTP2)
+    delete NativeModule._source.http2;
+
   NativeModule.require = function(id) {
     if (id === 'native_module') {
       return NativeModule;
@@ -524,8 +546,6 @@
     return NativeModule._source.hasOwnProperty(id);
   };
 
-  const config = process.binding('config');
-
   if (config.exposeInternals) {
     NativeModule.nonInternalExists = NativeModule.exists;
 
