@@ -1,5 +1,7 @@
 'use strict';
 
+const util = require('util');
+
 var _lazyConstants = null;
 
 function lazyConstants() {
@@ -155,6 +157,8 @@ function setupKillAndExit() {
       process._exiting = true;
       process.emit('exit', process.exitCode || 0);
     }
+
+    require('trace_mgr').onExitHandler(process.exitCode || 0); //ENABLE_TTD
     process.reallyExit(process.exitCode || 0);
   };
 
@@ -178,16 +182,14 @@ function setupKillAndExit() {
       }
     }
 
-    if (err) {
-      const errnoException = require('util')._errnoException;
-      throw errnoException(err, 'kill');
-    }
+    if (err)
+      throw util._errnoException(err, 'kill');
 
     return true;
   };
 }
 
-
+var ttdSigIntHandler = undefined;
 function setupSignalHandlers() {
   // Load events module in order to access prototype elements on process like
   // process.addListener.
@@ -206,14 +208,20 @@ function setupSignalHandlers() {
 
       wrap.unref();
 
-      wrap.onsignal = function() { process.emit(type); };
+      if (type === 'SIGINT' && global.enabledDiagnosticsTrace) {
+        //TODO: this is probably a little over complicated
+        //  -- should unify this and ttdSigIntHandler setup below
+        //  -- ttdSigIntHandler will emit event after it runs
+        wrap.onsignal = function() { }; //ENABLE_TTD
+      } else {
+        wrap.onsignal = function() { process.emit(type); };
+      }
 
       const signum = lazyConstants()[type];
       const err = wrap.start(signum);
       if (err) {
         wrap.close();
-        const errnoException = require('util')._errnoException;
-        throw errnoException(err, 'uv_signal_start');
+        throw util._errnoException(err, 'uv_signal_start');
       }
 
       signalWraps[type] = wrap;
@@ -226,6 +234,34 @@ function setupSignalHandlers() {
       delete signalWraps[type];
     }
   });
+
+  //ENABLE_TTD
+  if (global.enabledDiagnosticsTrace) {
+    (function() {
+      var Signal = process.binding('signal_wrap').Signal;
+      var wrap = new Signal();
+
+      wrap.unref();
+
+      var handler = require('trace_mgr').onSigIntHandler;
+      wrap.onsignal = function() {
+        handler(signalWraps.hasOwnProperty('SIGINT'));
+        process.emit('SIGINT');
+      };
+
+      const signum = lazyConstants()['SIGINT'];
+      const err = wrap.start(signum);
+      if (err) {
+        wrap.close();
+        const errnoException = require('util')._errnoException;
+        throw errnoException(err, 'uv_signal_start');
+      }
+
+      if (!ttdSigIntHandler) {
+        ttdSigIntHandler = wrap;
+      }
+    })();
+  }
 }
 
 
@@ -253,10 +289,9 @@ function setupChannel() {
 
 
 function setupRawDebug() {
-  const format = require('util').format;
   const rawDebug = process._rawDebug;
   process._rawDebug = function() {
-    rawDebug(format.apply(null, arguments));
+    rawDebug(util.format.apply(null, arguments));
   };
 }
 
