@@ -792,7 +792,7 @@ function complete(line, callback) {
     completeOn = match[1];
     var subdir = match[2] || '';
     filter = match[1];
-    var dir, files, f, name, base, ext, abs, subfiles, s;
+    var dir, files, f, name, base, ext, abs, subfiles, s, isDirectory;
     group = [];
     let paths = [];
 
@@ -821,23 +821,26 @@ function complete(line, callback) {
           // Exclude versioned names that 'npm' installs.
           continue;
         }
-        if (exts.indexOf(ext) !== -1) {
-          if (!subdir || base !== 'index') {
-            group.push(subdir + base);
-          }
-        } else {
-          abs = path.resolve(dir, name);
+        abs = path.resolve(dir, name);
+        try {
+          isDirectory = fs.statSync(abs).isDirectory();
+        } catch (e) {
+          continue;
+        }
+        if (isDirectory) {
+          group.push(subdir + name + '/');
           try {
-            if (fs.statSync(abs).isDirectory()) {
-              group.push(subdir + name + '/');
-              subfiles = fs.readdirSync(abs);
-              for (s = 0; s < subfiles.length; s++) {
-                if (indexRe.test(subfiles[s])) {
-                  group.push(subdir + name);
-                }
-              }
+            subfiles = fs.readdirSync(abs);
+          } catch (e) {
+            continue;
+          }
+          for (s = 0; s < subfiles.length; s++) {
+            if (indexRe.test(subfiles[s])) {
+              group.push(subdir + name);
             }
-          } catch (e) {}
+          }
+        } else if (exts.includes(ext) && (!subdir || base !== 'index')) {
+          group.push(subdir + base);
         }
       }
     }
