@@ -544,18 +544,9 @@ void AsyncWrap::Initialize(Local<Object> target,
       isolate,
       uid_fields_ptr,
       uid_fields_count * sizeof(*uid_fields_ptr));
-  Local<Float64Array> farray = Float64Array::New(uid_fields_ab, 0,
-                                                 uid_fields_count);
   FORCE_SET_TARGET_FIELD(target,
                          "async_uid_fields",
-                         farray);
-
-#if ENABLE_TTD_NODE
-  if (s_doTTRecord || s_doTTReplay) {
-    JsTTDNotifyLongLivedReferenceAdd(*farray);
-    env->async_hooks()->uid_fields_ttdRef = *farray;
-  }
-#endif
+                         Float64Array::New(uid_fields_ab, 0, uid_fields_count));
 
   Local<Object> constants = Object::New(isolate);
 #define SET_HOOKS_CONSTANT(name)                                              \
@@ -655,6 +646,8 @@ void AsyncWrap::EmitAsyncInit(Environment* env,
                               Local<String> type,
                               double async_id,
                               double trigger_id) {
+  CHECK(!object.IsEmpty());
+  CHECK(!type.IsEmpty());
   AsyncHooks* async_hooks = env->async_hooks();
 
   // Nothing to execute, so can continue normally.
