@@ -83,8 +83,7 @@ inline uint32_t* IsolateData::zero_fill_field() const {
 inline Environment::AsyncHooks::AsyncHooks(v8::Isolate* isolate)
     : isolate_(isolate),
       fields_(),
-      uid_fields_(),
-      uid_fields_ttdRef(nullptr) {
+      uid_fields_() {
   v8::HandleScope handle_scope(isolate_);
 
   // kAsyncUidCntr should start at 1 because that'll be the id the execution
@@ -135,10 +134,6 @@ inline void Environment::AsyncHooks::push_ids(double async_id,
                     uid_fields_[kCurrentTriggerId] });
   uid_fields_[kCurrentAsyncId] = async_id;
   uid_fields_[kCurrentTriggerId] = trigger_id;
-
-#if ENABLE_TTD_NODE
-  this->AsyncWrapId_TTDRecord();
-#endif
 }
 
 inline bool Environment::AsyncHooks::pop_ids(double async_id) {
@@ -168,11 +163,6 @@ inline bool Environment::AsyncHooks::pop_ids(double async_id) {
   ids_stack_.pop();
   uid_fields_[kCurrentAsyncId] = ids.async_id;
   uid_fields_[kCurrentTriggerId] = ids.trigger_id;
-
-#if ENABLE_TTD_NODE
-  this->AsyncWrapId_TTDRecord();
-#endif
-
   return !ids_stack_.empty();
 }
 
@@ -181,10 +171,6 @@ inline void Environment::AsyncHooks::clear_id_stack() {
     ids_stack_.pop();
   uid_fields_[kCurrentAsyncId] = 0;
   uid_fields_[kCurrentTriggerId] = 0;
-
-#if ENABLE_TTD_NODE
-  this->AsyncWrapId_TTDRecord();
-#endif
 }
 
 inline Environment::AsyncHooks::InitScope::InitScope(
@@ -273,6 +259,9 @@ inline void Environment::TickInfo::set_index(uint32_t value) {
 
 inline void Environment::AssignToContext(v8::Local<v8::Context> context) {
   context->SetAlignedPointerInEmbedderData(kContextEmbedderDataIndex, this);
+#if HAVE_INSPECTOR
+  inspector_agent()->ContextCreated(context);
+#endif  // HAVE_INSPECTOR
 }
 
 inline Environment* Environment::GetCurrent(v8::Isolate* isolate) {
@@ -286,18 +275,16 @@ inline Environment* Environment::GetCurrent(v8::Local<v8::Context> context) {
 
 inline Environment* Environment::GetCurrent(
     const v8::FunctionCallbackInfo<v8::Value>& info) {
-  ASSERT(info.Data()->IsExternal());
+  CHECK(info.Data()->IsExternal());
   return static_cast<Environment*>(info.Data().As<v8::External>()->Value());
 }
 
 template <typename T>
 inline Environment* Environment::GetCurrent(
     const v8::PropertyCallbackInfo<T>& info) {
-  ASSERT(info.Data()->IsExternal());
-  // XXX(bnoordhuis) Work around a g++ 4.9.2 template type inferrer bug
-  // when the expression is written as info.Data().As<v8::External>().
-  v8::Local<v8::Value> data = info.Data();
-  return static_cast<Environment*>(data.As<v8::External>()->Value());
+  CHECK(info.Data()->IsExternal());
+  return static_cast<Environment*>(
+      info.Data().template As<v8::External>()->Value());
 }
 
 inline Environment::Environment(IsolateData* isolate_data,
@@ -306,8 +293,6 @@ inline Environment::Environment(IsolateData* isolate_data,
       isolate_data_(isolate_data),
       async_hooks_(context->GetIsolate()),
       timer_base_(uv_now(isolate_data->event_loop())),
-      cares_query_last_ok_(true),
-      cares_is_servers_default_(true),
       using_domains_(false),
       printed_error_(false),
       trace_sync_io_(false),
@@ -318,7 +303,7 @@ inline Environment::Environment(IsolateData* isolate_data,
 #endif
       handle_cleanup_waiting_(0),
       http_parser_buffer_(nullptr),
-      fs_stats_field_array_(),
+      fs_stats_field_array_(nullptr),
       context_(context->GetIsolate(), context) {
   // We'll be creating new objects so make sure we've entered the context.
   v8::HandleScope handle_scope(isolate());
@@ -327,7 +312,6 @@ inline Environment::Environment(IsolateData* isolate_data,
   set_binding_cache_object(v8::Object::New(isolate()));
   set_module_load_list_array(v8::Array::New(isolate()));
 
-  RB_INIT(&cares_task_list_);
   AssignToContext(context);
 
   destroy_ids_list_.reserve(512);
@@ -336,8 +320,6 @@ inline Environment::Environment(IsolateData* isolate_data,
 inline Environment::~Environment() {
   v8::HandleScope handle_scope(isolate());
 
-  fs_stats_field_array_.Empty();
-
   context()->SetAlignedPointerInEmbedderData(kContextEmbedderDataIndex,
                                              nullptr);
 #define V(PropertyName, TypeName) PropertyName ## _.Reset();
@@ -347,6 +329,7 @@ inline Environment::~Environment() {
   delete[] heap_statistics_buffer_;
   delete[] heap_space_statistics_buffer_;
   delete[] http_parser_buffer_;
+  free(http2_state_buffer_);
 }
 
 inline v8::Isolate* Environment::isolate() const {
@@ -444,13 +427,7 @@ inline std::vector<double>* Environment::destroy_ids_list() {
 }
 
 inline double Environment::new_async_id() {
-  double res = ++async_hooks()->uid_fields()[AsyncHooks::kAsyncUidCntr];
-
-#if ENABLE_TTD_NODE
-  this->async_hooks()->AsyncWrapId_TTDRecord();
-#endif
-
-  return res;
+  return ++async_hooks()->uid_fields()[AsyncHooks::kAsyncUidCntr];
 }
 
 inline double Environment::current_async_id() {
@@ -465,21 +442,12 @@ inline double Environment::get_init_trigger_id() {
   double* uid_fields = async_hooks()->uid_fields();
   double tid = uid_fields[AsyncHooks::kInitTriggerId];
   uid_fields[AsyncHooks::kInitTriggerId] = 0;
-
-#if ENABLE_TTD_NODE
-  this->async_hooks()->AsyncWrapId_TTDRecord();
-#endif
-
   if (tid <= 0) tid = current_async_id();
   return tid;
 }
 
 inline void Environment::set_init_trigger_id(const double id) {
   async_hooks()->uid_fields()[AsyncHooks::kInitTriggerId] = id;
-
-#if ENABLE_TTD_NODE
-  this->async_hooks()->AsyncWrapId_TTDRecord();
-#endif
 }
 
 inline double* Environment::heap_statistics_buffer() const {
@@ -502,7 +470,6 @@ inline void Environment::set_heap_space_statistics_buffer(double* pointer) {
   heap_space_statistics_buffer_ = pointer;
 }
 
-
 inline char* Environment::http_parser_buffer() const {
   return http_parser_buffer_;
 }
@@ -512,51 +479,22 @@ inline void Environment::set_http_parser_buffer(char* buffer) {
   http_parser_buffer_ = buffer;
 }
 
-inline v8::Local<v8::Float64Array> Environment::fs_stats_field_array() const {
-  return v8::Local<v8::Float64Array>::New(isolate_, fs_stats_field_array_);
+inline http2::http2_state* Environment::http2_state_buffer() const {
+  return http2_state_buffer_;
 }
 
-inline void Environment::set_fs_stats_field_array(
-    v8::Local<v8::Float64Array> fields) {
-  CHECK_EQ(fs_stats_field_array_.IsEmpty(), true);  // Should be set only once.
-  fs_stats_field_array_ = v8::Global<v8::Float64Array>(isolate_, fields);
+inline void Environment::set_http2_state_buffer(http2::http2_state* buffer) {
+  CHECK_EQ(http2_state_buffer_, nullptr);  // Should be set only once.
+  http2_state_buffer_ = buffer;
 }
 
-inline Environment* Environment::from_cares_timer_handle(uv_timer_t* handle) {
-  return ContainerOf(&Environment::cares_timer_handle_, handle);
+inline double* Environment::fs_stats_field_array() const {
+  return fs_stats_field_array_;
 }
 
-inline uv_timer_t* Environment::cares_timer_handle() {
-  return &cares_timer_handle_;
-}
-
-inline ares_channel Environment::cares_channel() {
-  return cares_channel_;
-}
-
-// Only used in the call to ares_init_options().
-inline ares_channel* Environment::cares_channel_ptr() {
-  return &cares_channel_;
-}
-
-inline bool Environment::cares_query_last_ok() {
-  return cares_query_last_ok_;
-}
-
-inline void Environment::set_cares_query_last_ok(bool ok) {
-  cares_query_last_ok_ = ok;
-}
-
-inline bool Environment::cares_is_servers_default() {
-  return cares_is_servers_default_;
-}
-
-inline void Environment::set_cares_is_servers_default(bool is_default) {
-  cares_is_servers_default_ = is_default;
-}
-
-inline node_ares_task_list* Environment::cares_task_list() {
-  return &cares_task_list_;
+inline void Environment::set_fs_stats_field_array(double* fields) {
+  CHECK_EQ(fs_stats_field_array_, nullptr);  // Should be set only once.
+  fs_stats_field_array_ = fields;
 }
 
 inline IsolateData* Environment::isolate_data() const {
