@@ -1122,7 +1122,7 @@ napi_status napi_create_string_utf16(napi_env env,
   return napi_ok;
 }
 
-napi_status napi_create_number(napi_env env,
+napi_status napi_create_double(napi_env env,
                                double value,
                                napi_value* result) {
   CHECK_ARG(result);
@@ -1130,6 +1130,32 @@ napi_status napi_create_number(napi_env env,
   return napi_ok;
 }
 
+napi_status napi_create_int32(napi_env env,
+                              int32_t value,
+                              napi_value* result) {
+  CHECK_ARG(result);
+  CHECK_JSRT(JsIntToNumber(value, reinterpret_cast<JsValueRef*>(result)));
+  return napi_ok;
+}
+
+napi_status napi_create_uint32(napi_env env,
+                               uint32_t value,
+                               napi_value* result) {
+  CHECK_ARG(result);
+  CHECK_JSRT(JsDoubleToNumber(static_cast<double>(value),
+                              reinterpret_cast<JsValueRef*>(result)));
+  return napi_ok;
+}
+
+napi_status napi_create_int64(napi_env env,
+                              int64_t value,
+                              napi_value* result) {
+  CHECK_ARG(result);
+  CHECK_JSRT(JsDoubleToNumber(static_cast<double>(value),
+                              reinterpret_cast<JsValueRef*>(result)));
+  return napi_ok;
+}
+
 napi_status napi_get_boolean(napi_env env, bool value, napi_value* result) {
   CHECK_ARG(result);
   CHECK_JSRT(JsBoolToBoolean(value, reinterpret_cast<JsValueRef*>(result)));
@@ -2332,6 +2358,19 @@ napi_status napi_get_version(napi_env env, uint32_t* result) {
   return napi_ok;
 }
 
+napi_status napi_get_node_version(napi_env env,
+                                  const napi_node_version** result) {
+  CHECK_ARG(result);
+  static const napi_node_version version = {
+    NODE_MAJOR_VERSION,
+    NODE_MINOR_VERSION,
+    NODE_PATCH_VERSION,
+    NODE_RELEASE
+  };
+  *result = &version;
+  return napi_ok;
+}
+
 namespace uvimpl {
 
 napi_status ConvertUVErrorCode(int code) {
