@@ -1,9 +1,7 @@
 #include "myobject.h"
 #include "../common.h"
 
-extern size_t finalize_count;
-
-static napi_value CreateObject(napi_env env, napi_callback_info info) {
+napi_value CreateObject(napi_env env, napi_callback_info info) {
   size_t argc = 1;
   napi_value args[1];
   NAPI_CALL(env, napi_get_cb_info(env, info, &argc, args, nullptr, nullptr));
@@ -14,7 +12,7 @@ static napi_value CreateObject(napi_env env, napi_callback_info info) {
   return instance;
 }
 
-static napi_value Add(napi_env env, napi_callback_info info) {
+napi_value Add(napi_env env, napi_callback_info info) {
   size_t argc = 2;
   napi_value args[2];
   NAPI_CALL(env, napi_get_cb_info(env, info, &argc, args, nullptr, nullptr));
@@ -31,25 +29,16 @@ static napi_value Add(napi_env env, napi_callback_info info) {
   return sum;
 }
 
-static napi_value FinalizeCount(napi_env env, napi_callback_info info) {
-  napi_value return_value;
-  NAPI_CALL(env, napi_create_uint32(env, finalize_count, &return_value));
-  return return_value;
-}
-
-static napi_value Init(napi_env env, napi_value exports) {
+void Init(napi_env env, napi_value exports, napi_value module, void* priv) {
   MyObject::Init(env);
 
   napi_property_descriptor desc[] = {
     DECLARE_NAPI_PROPERTY("createObject", CreateObject),
     DECLARE_NAPI_PROPERTY("add", Add),
-    DECLARE_NAPI_PROPERTY("finalizeCount", FinalizeCount),
   };
 
-  NAPI_CALL(env,
-      napi_define_properties(env, exports, sizeof(desc) / sizeof(*desc), desc));
-
-  return exports;
+  NAPI_CALL_RETURN_VOID(env,
+    napi_define_properties(env, exports, sizeof(desc) / sizeof(*desc), desc));
 }
 
 NAPI_MODULE(addon, Init)
