@@ -3,8 +3,11 @@ const common = require('../common');
 const assert = require('assert');
 const dns = require('dns');
 const net = require('net');
+const util = require('util');
 const isIPv4 = net.isIPv4;
 
+common.crashOnUnhandledRejection();
+
 let running = false;
 const queue = [];
 
@@ -184,3 +187,13 @@ TEST(function test_lookupservice_ip_ipv4(done) {
 
   checkWrap(req);
 });
+
+TEST(function test_lookupservice_ip_ipv4_promise(done) {
+  util.promisify(dns.lookupService)('127.0.0.1', 80)
+    .then(common.mustCall(({hostname, service}) => {
+      assert.strictEqual(typeof hostname, 'string');
+      assert(hostname.length > 0);
+      assert(['http', 'www', '80'].includes(service));
+      done();
+    }));
+});
