@@ -7,7 +7,7 @@ const assert = require('assert');
 const crypto = require('crypto');
 
 function testCipher1(key, iv) {
-  // Test encyrption and decryption with explicit key and iv
+  // Test encryption and decryption with explicit key and iv
   const plaintext =
           '32|RmVZZkFUVmpRRkp0TmJaUm56ZU9qcnJkaXNNWVNpTTU*|iXmckfRWZBGWWELw' +
           'eCBsThSsfUHLeRe0KCsK8ooHgxie0zOINpXxfZi/oNG7uq9JWFVCk70gfzQH8ZUJ' +
@@ -20,8 +20,7 @@ function testCipher1(key, iv) {
   let txt = decipher.update(ciph, 'hex', 'utf8');
   txt += decipher.final('utf8');
 
-  assert.strictEqual(txt, plaintext,
-                     `encryption/decryption with key ${key} and iv ${iv}`);
+  assert.strictEqual(txt, plaintext, 'encryption/decryption with key and iv');
 
   // streaming cipher interface
   // NB: In real life, it's not guaranteed that you can get all of it
@@ -35,13 +34,12 @@ function testCipher1(key, iv) {
   dStream.end(ciph);
   txt = dStream.read().toString('utf8');
 
-  assert.strictEqual(txt, plaintext,
-                     `streaming cipher with key ${key} and iv ${iv}`);
+  assert.strictEqual(txt, plaintext, 'streaming cipher iv');
 }
 
 
 function testCipher2(key, iv) {
-  // Test encyrption and decryption with explicit key and iv
+  // Test encryption and decryption with explicit key and iv
   const plaintext =
           '32|RmVZZkFUVmpRRkp0TmJaUm56ZU9qcnJkaXNNWVNpTTU*|iXmckfRWZBGWWELw' +
           'eCBsThSsfUHLeRe0KCsK8ooHgxie0zOINpXxfZi/oNG7uq9JWFVCk70gfzQH8ZUJ' +
@@ -54,28 +52,7 @@ function testCipher2(key, iv) {
   let txt = decipher.update(ciph, 'buffer', 'utf8');
   txt += decipher.final('utf8');
 
-  assert.strictEqual(txt, plaintext,
-                     `encryption/decryption with key ${key} and iv ${iv}`);
-}
-
-
-function testCipher3(key, iv) {
-  // Test encryption and decryption with explicit key and iv.
-  // AES Key Wrap test vector comes from RFC3394
-  const plaintext = Buffer.from('00112233445566778899AABBCCDDEEFF', 'hex');
-
-  const cipher = crypto.createCipheriv('id-aes128-wrap', key, iv);
-  let ciph = cipher.update(plaintext, 'utf8', 'buffer');
-  ciph = Buffer.concat([ciph, cipher.final('buffer')]);
-  const ciph2 = Buffer.from('1FA68B0A8112B447AEF34BD8FB5A7B829D3E862371D2CFE5',
-                            'hex');
-  assert(ciph.equals(ciph2));
-  const decipher = crypto.createDecipheriv('id-aes128-wrap', key, iv);
-  let deciph = decipher.update(ciph, 'buffer');
-  deciph = Buffer.concat([deciph, decipher.final()]);
-
-  assert(deciph.equals(plaintext),
-         `encryption/decryption with key ${key} and iv ${iv}`);
+  assert.strictEqual(txt, plaintext, 'encryption/decryption with key and iv');
 }
 
 testCipher1('0123456789abcd0123456789', '12345678');
@@ -84,11 +61,6 @@ testCipher1(Buffer.from('0123456789abcd0123456789'), '12345678');
 testCipher1(Buffer.from('0123456789abcd0123456789'), Buffer.from('12345678'));
 testCipher2(Buffer.from('0123456789abcd0123456789'), Buffer.from('12345678'));
 
-if (!common.hasFipsCrypto) {
-  testCipher3(Buffer.from('000102030405060708090A0B0C0D0E0F', 'hex'),
-              Buffer.from('A6A6A6A6A6A6A6A6', 'hex'));
-}
-
 // Zero-sized IV should be accepted in ECB mode.
 crypto.createCipheriv('aes-128-ecb', Buffer.alloc(16), Buffer.alloc(0));
 
