@@ -59,13 +59,16 @@ assert.throws(() => {
 }, /^TypeError: First argument must be a string or Buffer$/);
 
 // addTrailers
-assert.throws(() => {
-  const outgoingMessage = new OutgoingMessage();
-  outgoingMessage.addTrailers();
-}, common.engineSpecificMessage({
-        v8: /^TypeError: Cannot convert undefined or null to object$/,
-        chakracore: /^TypeError: Object expected/
-}));
+assert.throws(
+  () => {
+    const outgoingMessage = new OutgoingMessage();
+    outgoingMessage.addTrailers();
+  },
+  common.engineSpecificMessage({
+    v8: /^TypeError: Cannot convert undefined or null to object$/,
+    chakracore: /^TypeError: Object expected/
+  })
+);
 
 assert.throws(() => {
   const outgoingMessage = new OutgoingMessage();
