@@ -1,5 +1,6 @@
 'use strict';
 const common = require('../common');
+const os = require('os');
 if (!(common.hasIntl && common.hasSmallICU))
   common.skip('missing Intl');
 
@@ -8,7 +9,7 @@ const { spawnSync } = require('child_process');
 
 const expected =
     'could not initialize ICU (check NODE_ICU_DATA or ' +
-    '--icu-data-dir parameters)' + (common.isWindows ? '\r\n' : '\n');
+    `--icu-data-dir parameters)${os.EOL}`;
 
 {
   const child = spawnSync(process.execPath, ['--icu-data-dir=/', '-e', '0']);
