@@ -5,6 +5,10 @@ const common = require('../common');
 if (common.isSunOS)
   common.skip('unreliable on SunOS');
 
+if (common.isChakraEngine)
+  common.skip('This test is disabled for chakra engine because debugger ' +
+              'support is not implemented yet.');
+
 const assert = require('assert');
 const path = require('path');
 const childProcess = require('child_process');
@@ -104,7 +108,7 @@ replProc.on('close', function(code) {
 // also test that duplicated preload only gets loaded once
 childProcess.exec(
   `"${nodeBinary}" ${preloadOption([fixtureA])}-e "console.log('hello');" ${
-  preloadOption([fixtureA, fixtureB])}`,
+    preloadOption([fixtureA, fixtureB])}`,
   function(err, stdout, stderr) {
     assert.ifError(err);
     assert.strictEqual(stdout, 'A\nB\nhello\n');
@@ -125,7 +129,7 @@ interactive.stdin.write('process.exit()\n');
 
 childProcess.exec(
   `"${nodeBinary}" --require "${fixture('cluster-preload.js')}" "${
-  fixture('cluster-preload-test.js')}"`,
+    fixture('cluster-preload-test.js')}"`,
   function(err, stdout, stderr) {
     assert.ifError(err);
     assert.ok(/worker terminated with code 43/.test(stdout));
