@@ -3,7 +3,7 @@ const common = require('../common');
 const assert = require('assert');
 
 const expected_keys = ['ares', 'http_parser', 'modules', 'node',
-                       'uv', 'zlib'];
+                       'uv', 'zlib', 'nghttp2'];
 expected_keys.push(process.jsEngine);
 
 if (common.hasCrypto) {
@@ -30,6 +30,6 @@ assert(commonTemplate.test(process.versions.node));
 assert(commonTemplate.test(process.versions.uv));
 assert(commonTemplate.test(process.versions.zlib));
 
-assert(/^\d+\.\d+\.\d+(?:\.\d+)?(?: \(candidate\))?$/.test(
-  process.versions[process.jsEngine || 'v8']));
+assert(/^\d+\.\d+\.\d+(?:\.\d+)?(?: \(candidate\))?$/
+  .test(process.versions[process.jsEngine || 'v8']));
 assert(/^\d+$/.test(process.versions.modules));
