@@ -146,7 +146,7 @@ function error_test() {
     // But passing the same string to eval() should throw
     { client: client_unix, send: 'eval("function test_func() {")',
       expect: common.engineSpecificMessage({
-        v8: /\bSyntaxError: Unexpected end of input/,
+        v8: /^SyntaxError: Unexpected end of input/,
         chakracore: /^SyntaxError: Expected '}'/})
     },
     // Can handle multiline template literals
@@ -178,34 +178,34 @@ function error_test() {
     // should throw
     { client: client_unix, send: 'JSON.parse(\'{invalid: \\\'json\\\'}\');',
       expect: common.engineSpecificMessage({
-        v8: /\bSyntaxError: Unexpected token i/,
+        v8: /^SyntaxError: Unexpected token i/,
         chakracore: /^SyntaxError: JSON\.parse Error: Invalid character/})
     },
     // end of input to JSON.parse error is special case of syntax error,
     // should throw
     { client: client_unix, send: 'JSON.parse(\'066\');',
       expect: common.engineSpecificMessage({
-        v8: /\bSyntaxError: Unexpected number/,
+        v8: /^SyntaxError: Unexpected number/,
         chakracore:  /^SyntaxError: JSON\.parse Error: Invalid number/})
     },
     // should throw
     { client: client_unix, send: 'JSON.parse(\'{\');',
       expect: common.engineSpecificMessage({
-        v8: /\bSyntaxError: Unexpected end of JSON input/,
+        v8: /^SyntaxError: Unexpected end of JSON input/,
         chakracore: /^SyntaxError: JSON\.parse Error: Invalid character/})
     },
     // invalid RegExps are a special case of syntax error,
     // should throw
     { client: client_unix, send: '/(/;',
       expect: common.engineSpecificMessage({
-        v8: /\bSyntaxError: Invalid regular expression:/,
+        v8: /^SyntaxError: Invalid regular expression:/,
         chakracore: /^SyntaxError: Expected '\)' in regular expression/})
     },
     // invalid RegExp modifiers are a special case of syntax error,
     // should throw (GH-4012)
     { client: client_unix, send: 'new RegExp("foo", "wrong modifier");',
       expect: common.engineSpecificMessage({
-        v8: /\bSyntaxError: Invalid flags supplied to RegExp constructor/,
+        v8: /^SyntaxError: Invalid flags supplied to RegExp constructor/,
         chakracore: /^SyntaxError: Syntax error in regular expression/})
     },
     // strict mode syntax errors should be caught (GH-5178)
@@ -213,28 +213,28 @@ function error_test() {
       send: '(function() { "use strict"; return 0755; })()',
       expect: common.engineSpecificMessage({
         v8: /\bSyntaxError: Octal literals are not allowed in strict mode/,
-        chakracore: /^SyntaxError: Octal numeric literals and escape characters not allowed in strict mode/}) // eslint-disable-line max-len
+        chakracore: /^SyntaxError: Octal numeric literals and escape characters not allowed in strict mode/})
     },
     {
       client: client_unix,
       send: '(function(a, a, b) { "use strict"; return a + b + c; })()',
       expect: common.engineSpecificMessage({
-        v8: /\bSyntaxError: Duplicate parameter name not allowed in this context/, // eslint-disable-line max-len
-        chakracore: /^SyntaxError: Duplicate formal parameter names not allowed in strict mode/}) // eslint-disable-line max-len
+        v8: /\bSyntaxError: Duplicate parameter name not allowed in this context/,
+        chakracore: /^SyntaxError: Duplicate formal parameter names not allowed in strict mode/})
     },
     {
       client: client_unix,
       send: '(function() { "use strict"; with (this) {} })()',
       expect: common.engineSpecificMessage({
         v8: /\bSyntaxError: Strict mode code may not include a with statement/,
-        chakracore: /^SyntaxError: 'with' statements are not allowed in strict mode/}) // eslint-disable-line max-len
+        chakracore: /^SyntaxError: 'with' statements are not allowed in strict mode/})
     },
     {
       client: client_unix,
       send: '(function() { "use strict"; var x; delete x; })()',
       expect: common.engineSpecificMessage({
         v8: /\bSyntaxError: Delete of an unqualified identifier in strict mode/,
-        chakracore: /^SyntaxError: Calling delete on expression not allowed in strict mode/}) // eslint-disable-line max-len
+        chakracore: /^SyntaxError: Calling delete on expression not allowed in strict mode/})
     },
     { client: client_unix,
       send: '(function() { "use strict"; eval = 17; })()',
@@ -246,7 +246,7 @@ function error_test() {
       client: client_unix,
       send: '(function() { "use strict"; if (true) function f() { } })()',
       expect: common.engineSpecificMessage({
-        v8: /\bSyntaxError: In strict mode code, functions can only be declared at top level or inside a block\./, // eslint-disable-line max-len
+        v8: /\bSyntaxError: In strict mode code, functions can only be declared at top level or inside a block\./,
         chakracore: /^SyntaxError: Syntax error/})
     },
     // Named functions can be used:
