@@ -13,11 +13,12 @@ const request = {
   response: require(path.join(common.fixturesDir, 'url-tests'))
 };
 
-/* eslint-disable */
-/* WPT Refs:
+/* The following tests are copied from WPT. Modifications to them should be
+   upstreamed first. Refs:
    https://github.com/w3c/web-platform-tests/blob/8791bed/url/url-origin.html
    License: http://www.w3.org/Consortium/Legal/2008/04-testsuite-copyright.html
 */
+/* eslint-disable */
 function runURLOriginTests() {
   // var setup = async_test("Loading data…")
   // setup.step(function() {
