@@ -118,14 +118,9 @@ startURLSettersTests()
       assert.throws(() => url[name] = obj,
                     /^Error: toString$/,
                     `url.${name} = { toString() { throw ... } }`);
-      assert.throws(
-        () => url[name] = sym,
-        common.engineSpecificMessage({
-          v8: /^TypeError: Cannot convert a Symbol value to a string$/,
-          chakracore: /^TypeError: Object doesn't support property or method 'ToString'$/,
-        }),
-        `url.${name} = ${String(sym)}`
-      );
+      assert.throws(() => url[name] = sym,
+                    /^TypeError: Cannot convert a Symbol value to a string$/,
+                    `url.${name} = ${String(sym)}`);
     }
   }
 }
