@@ -1,3 +1,24 @@
+// Copyright Joyent, Inc. and other Node contributors.
+//
+// Permission is hereby granted, free of charge, to any person obtaining a
+// copy of this software and associated documentation files (the
+// "Software"), to deal in the Software without restriction, including
+// without limitation the rights to use, copy, modify, merge, publish,
+// distribute, sublicense, and/or sell copies of the Software, and to permit
+// persons to whom the Software is furnished to do so, subject to the
+// following conditions:
+//
+// The above copyright notice and this permission notice shall be included
+// in all copies or substantial portions of the Software.
+//
+// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
+// OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
+// MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN
+// NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM,
+// DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR
+// OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE
+// USE OR OTHER DEALINGS IN THE SOFTWARE.
+
 'use strict';
 const common = require('../common');
 const assert = require('assert');
@@ -8,7 +29,6 @@ const TIMER = 200;
 const SLEEP = 2000;
 
 const start = Date.now();
-const execOpts = { encoding: 'utf8', shell: true };
 let err;
 let caught = false;
 
@@ -33,8 +53,7 @@ try {
   assert.strictEqual(e.errno, 'ETIMEDOUT');
   err = e;
 } finally {
-  assert.strictEqual(ret, undefined,
-                     `should not have a return value, received ${ret}`);
+  assert.strictEqual(ret, undefined, 'we should not have a return value');
   assert.strictEqual(caught, true, 'execSync should throw');
   const end = Date.now() - start;
   assert(end < SLEEP);
@@ -55,11 +74,11 @@ cmd = `"${process.execPath}" -e "console.log('${msg}');"`;
 ret = execSync(cmd);
 
 assert.strictEqual(ret.length, msgBuf.length);
-assert.deepStrictEqual(ret, msgBuf);
+assert.deepStrictEqual(ret, msgBuf, 'execSync result buffer should match');
 
 ret = execSync(cmd, { encoding: 'utf8' });
 
-assert.strictEqual(ret, `${msg}\n`);
+assert.strictEqual(ret, `${msg}\n`, 'execSync encoding result should match');
 
 const args = [
   '-e',
@@ -71,10 +90,10 @@ assert.deepStrictEqual(ret, msgBuf);
 
 ret = execFileSync(process.execPath, args, { encoding: 'utf8' });
 
-assert.strictEqual(ret, `${msg}\n`);
+assert.strictEqual(ret, `${msg}\n`,
+                   'execFileSync encoding result should match');
 
-// Verify that the cwd option works.
-// See https://github.com/nodejs/node-v0.x-archive/issues/7824.
+// Verify that the cwd option works - GH #7824
 {
   const cwd = common.rootDir;
   const cmd = common.isWindows ? 'echo %cd%' : 'pwd';
@@ -83,8 +102,7 @@ assert.strictEqual(ret, `${msg}\n`);
   assert.strictEqual(response.toString().trim(), cwd);
 }
 
-// Verify that stderr is not accessed when stdio = 'ignore'.
-// See https://github.com/nodejs/node-v0.x-archive/issues/7966.
+// Verify that stderr is not accessed when stdio = 'ignore' - GH #7966
 {
   assert.throws(function() {
     execSync('exit -1', {stdio: 'ignore'});
@@ -101,13 +119,8 @@ assert.strictEqual(ret, `${msg}\n`);
     const msg = `Command failed: ${process.execPath} ${args.join(' ')}`;
 
     assert(err instanceof Error);
-    assert.strictEqual(err.message.trim(), msg);
+    assert.strictEqual(err.message, msg);
     assert.strictEqual(err.status, 1);
     return true;
   });
 }
-
-// Verify the shell option works properly
-assert.doesNotThrow(() => {
-  execFileSync(process.execPath, [], execOpts);
-});
