@@ -1,10 +1,9 @@
 'use strict';
 
-require('../common');
-const fixtures = require('../common/fixtures');
+const common = require('../common');
 const assert = require('assert');
 const execFile = require('child_process').execFile;
-const warnmod = require.resolve(fixtures.path('warnings.js'));
+const warnmod = require.resolve(`${common.fixturesDir}/warnings.js`);
 const node = process.execPath;
 
 const normal = [warnmod];
@@ -32,5 +31,5 @@ execFile(node, traceWarn, function(er, stdout, stderr) {
   assert.strictEqual(er, null);
   assert.strictEqual(stdout, '');
   assert(warningMessage.test(stderr));
-  assert(/at Object\.<anonymous>\s\(.+warnings.js:3:9\)/.test(stderr));
+  assert(/at Object\.<anonymous>\s\(.+warnings\.js:3:9\)/.test(stderr));
 });
